# -*- coding: utf-8 -*-
"""
uuid helper
"""
import uuid
import time
import random


def create_uuid():
    """
    Generate random string
    """
    return uuid.uuid4()


def create_short_uuid():
    """
    Generate random integer
    """
    result = str(random.randint(0, 999999999))
    return result


def create_short_timestamp_uuid():
    """
    Generate short uuid plus timestamp
    """
    short_uuid = create_short_uuid()
    return "{}{}".format(short_uuid, time.time())



# -*- coding: utf-8 -*-
"""
Poco Element verification
"""
import flybirds.core.plugin.plugins.default.ui_driver.poco.poco_attr as pa
import flybirds.core.plugin.plugins.default.ui_driver.poco.poco_text as pt
import flybirds.utils.verify_helper as verify_helper


def ele_text_is(
    poco,
    selector_str,
    target_str,
    optional,
    deal_method=None,
    params_deal_module=None,
):
    """
    determine whether the element is the expected value
    """
    ele_str = pt.get_ele_text_replace_space(
        poco, selector_str, optional, deal_method, params_deal_module
    )
    verify_helper.text_equal(target_str, ele_str)


def ele_text_contains(
    poco,
    selector_str,
    target_str,
    optional,
    deal_method=None,
    params_deal_module=None,
):
    """
    determine whether the element contains
    """
    ele_str = pt.get_ele_text_replace_space(
        poco, selector_str, optional, deal_method, params_deal_module
    )
    verify_helper.text_container(target_str, ele_str)


def ele_attr_is(
    poco,
    selector_str,
    optional,
    target_attr,
    target_attr_value,
    deal_method,
    params_deal_module,
):
    """
    determine whether the specified attribute of the element is the expected
    value.
    """
    ele_attr = pa.get_ele_attr(
        poco,
        selector_str,
        optional,
        target_attr,
        deal_method,
        params_deal_module,
    )
    verify_helper.attr_equal(target_attr_value, ele_attr)



# -*- coding: utf-8 -*-
"""
manage user py code
"""
import sys
import os


class ScriptImportManage:
    """
    use user py module code
    """

    def __init__(self):
        sys.path.insert(1, os.path.join(os.getcwd(), "pscript"))
        params_deal_path = os.path.join(
            os.getcwd(), "pscript", "params_deal.py"
        )
        if os.path.exists(params_deal_path):
            params_deal_import_path = "params_deal"
            self.params_deal = __import__(params_deal_import_path)
        app_operation_path = os.path.join(
            os.getcwd(), "pscript", "app", "operation.py"
        )
        if os.path.exists(app_operation_path):
            app_operation_import_path = "app.operation"
            app_module = __import__(app_operation_import_path)
            self.app_operation = getattr(app_module, "operation")
        dsl_hook_path = os.path.join(os.getcwd(), "pscript", "dsl", "hook.py")
        if os.path.exists(dsl_hook_path):
            dsl_hook_import_path = "dsl.hook"
            dsl_module = __import__(dsl_hook_import_path)
            self.dsl_hook = getattr(dsl_module, "hook")



# -*- coding: utf-8 -*-
"""
Step implement of element input.
"""
import flybirds.core.global_resource as gr
import flybirds.utils.dsl_helper as dsl_helper

from flybirds.core.global_context import GlobalContext as g_Context


def ele_input(context, param1, param2):
    poco_instance = gr.get_value("pocoInstance")
    param1_dict = dsl_helper.params_to_dic(param1)

    selector_str = param1_dict["selector"]
    optional = {}
    if "path" in param1_dict.keys():
        optional["path"] = param1_dict["path"]
    elif "multiSelector" in param1_dict.keys():
        optional["multiSelector"] = param1_dict["multiSelector"]
    if "timeout" in param1_dict.keys():
        optional["timeout"] = float(param1_dict["timeout"])
    else:
        optional["timeout"] = gr.get_frame_config_value("wait_ele_timeout", 10)

    param2_dict = dsl_helper.params_to_dic(param2, "inputStr")
    input_str = param2_dict["inputStr"]
    use_poco_input = False
    if "pocoInput" in param2_dict.keys():
        use_poco_input = True if param2_dict["pocoInput"] else False
    else:
        use_poco_input = gr.get_frame_config_value("use_poco_input", False)
    after_input_wait = None
    if "afterInputWait" in param2_dict.keys():
        after_input_wait = float(param2_dict["afterInputWait"])
    else:
        after_input_wait = gr.get_frame_config_value("after_input_wait", 1)

    if use_poco_input:
        g_Context.element.air_bdd_input(
            poco_instance, selector_str, optional, input_str, after_input_wait
        )
    else:
        g_Context.element.air_bdd_click(
            poco_instance, selector_str, optional, None, None, None
        )
        g_Context.element.str_input(input_str, after_input_wait)



# -*- coding: utf-8 -*-
"""
ui driver
"""
from flybirds.core.plugin.plugins.default.ui_driver.poco.poco_manage import \
    poco_init
from flybirds.core.plugin.plugins.default.ui_driver.poco.poco_screen import \
    air_bdd_screen_size

__open__ = ["UIDriver"]


class UIDriver:
    """
    ui driver
    """
    name = "ios_ui_driver"

    def poco_init(self):
        return poco_init()

    def air_bdd_screen_size(self, dr_instance):
        return air_bdd_screen_size(dr_instance)



# -*- coding: utf-8 -*-
globalization = {
    "en": {
        "rerun failed scenario": u"rerun failed scenario",
        "information association of failed operation":
            "information association of failed operation, run the {} time"
            " :[{}]",
        "start record": "start record",
        "stop record": "stop record",
        "rank": "rank",
        "parent": "parent",
        "children": "children",
        "sibling": "sibling",
        "offsprings": "offsprings",
        "and": "and",
        "up": "up",
        "down": "down",
        "left": "left",
        "right": "right",
        "modal_list": ["name=同意并继续",
                       "name=使用App时允许",
                       "name=允许",
                       "name=common update X",
                       "name=我知道了",
                       "text=同意并继续",
                       "text=使用App时允许",
                       "text=允许",
                       "text=common update X",
                       "text=我知道了",
                       "text=System UI isn't responding",
                       "text=Wait"]
    },
    "zh-CN": {
        "rerun failed scenario": u"\u5931\u8d25\u91cd\u65b0\u8fd0\u884c",
        "information association of failed operation":
            "失败运行的信息关联,运行第{}次:[{}]",
        "start record": u"开始录屏",
        "stop record": u"结束录屏",
        "rank": u"第",
        "parent": u"父节点",
        "children": u"孩子",
        "sibling": u"兄弟",
        "offsprings": u"后代",
        "and": u"并且",
        "up": u"上",
        "down": u"下",
        "left": u"左",
        "right": u"右",
        "modal_list": ["name=同意并继续",
                       "name=使用App时允许",
                       "name=允许",
                       "name=common update X",
                       "name=我知道了",
                       "text=同意并继续",
                       "text=使用App时允许",
                       "text=允许",
                       "text=common update X",
                       "text=我知道了",
                       "text=System UI isn't responding",
                       "text=Wait"]
    },
}

step_language = {
    "en": {
        "install app[{param}]": ["install app[{param}]"],
        "start app[{param}]": ["start app[{param}]"],
    },
    "zh-CN": {
        "install app[{param}]": ["安装APP[{param}]"],
        "delete app[{param}]": ["删除APP[{param}]"],
        "start app[{param}]": ["启动APP[{param}]"],
        "restart app": ["重启App"],
        "close app": ["关闭App"],
        "init device[{param}]": ["设备初始化[{param}]"],
        "connect device[{param}]": ["连接设备[{param}]"],
        "start recording timeout[{param}]": ["开始录屏超时[{param}]"],
        "start record": ["开始录屏"],
        "stop record": ["结束录屏"],
        "go to url[{param}]": ["跳转到[{param}]", "跳转页面到[{param}]"],
        "return to previous page": ["返回上一页"],
        "go to home page": ["回到首页"],
        "logon account[{param1}]password[{param2}]": [
            "登录账号[{param1}]密码[{param2}]",
            "登陆账号[{param1}]密码[{param2}]",
        ],
        "logout": ["退出登陆", "退出登录"],
        "wait[{param}]seconds": ["等待[{param}]秒"],
        "screenshot": ["全屏截图"],
        "information association of failed operation, run the {param1} time"
        " :[{param2}]": ["失败运行的信息关联,运行第{param1}次:[{param2}]"],
        "text[{param1}]property[{param2}]is {param3}": [
            "文案[{param1}]的属性[{param2}]为{param3}"
        ],
        "element[{param1}]property[{param2}]is {param3}": [
            "元素[{param1}]的属性[{param2}]为{param3}"
        ],
        "click[{param}]": ["点击[{param}]"],
        "click text[{param}]": ["点击文案[{param}]"],
        "click position[{x},{y}]": ["点击屏幕位置[{x},{y}]"],
        "in[{param1}]input[{param2}]": ["在[{param1}]中输入[{param2}]"],
        "element[{param1}]position not change in[{param2}]seconds": [
            "元素[{param1}]位置[{param2}]秒内未变动"
        ],
        "[{param1}]slide to {param2} distance[{param3}]": [
            "[{param1}]向{param2}滑动[{param3}]"],
        "slide to {param1} distance[{param2}]": ["全屏向{param1}滑动[{param2}]"],
        "exist text[{param}]": ["存在[{param}]的文案"],
        "not exist text[{param}]": ["不存在[{param}]的文案"],
        "text[{param}]disappear": ["文案[{param}]消失"],
        "exist[{param}]element": ["存在[{param}]的元素"],
        "not exist element[{param}]": ["不存在[{param}]的元素"],
        "element[{param}]disappear": ["元素[{param}]消失"],
        "the text of element[{param1}]is[{param2}]": [
            "[{param1}]的文案为[{param2}]"
        ],
        "the text of element[{param1}]include[{param2}]": [
            "[{param1}]的文案包含[{param2}]"
        ],
        "page rendering complete appears element[{param}]": [
            "页面渲染完成出现元素[{param}]"],
        "existing element[{param}]": ["存在元素[{param}]"],

        "in[{param1}]from {param2} find[{param3}]element": [
            "在[{param1}]中向{param2}查找[{param3}]的元素"
        ],
        "from {param1} find[{param2}]element": ["向{param1}查找[{param2}]的元素"],
    },
}



# -*- coding: utf-8 -*-
"""
ios element core api implement
"""

from flybirds.core.plugin.plugins.default.base_element import BaseElement

__open__ = ["Element"]


class Element(BaseElement):
    """IOS Element Class"""

    name = "ios_element"

    def ui_driver_init(self):
        """
        Initialize the poco object
         :return:
        """
        from poco.drivers.ios import iosPoco

        ios_poco = iosPoco()
        return ios_poco



# -*- coding: utf-8 -*-
"""
Text apis
"""
import re
import time

import flybirds.core.plugin.plugins.default.ui_driver.poco.poco_ele as poco_ele
import flybirds.core.plugin.plugins.default.ui_driver.poco.poco_manage as pm


def get_ele_text_replace_space(
        poco, selector_str, optional, deal_method_name, params_deal_module
):
    """
    get the text of the element, and after the text is obtained,
    you can choose whether to process it by a custom method
    """
    pattern = re.compile(r"\s+")
    poco_ele.wait_exists(poco, selector_str, optional)
    poco_object = pm.create_poco_object_by_dsl(
        poco, selector_str, optional
    )
    ele_str = poco_object.get_text()
    if ele_str is None:
        ele_str = poco_object.attr('label')
    handled_ele_str = re.sub(pattern, "", ele_str.replace(u"\u200b", ""))
    if not (deal_method_name is None):
        deal_method = getattr(params_deal_module, deal_method_name)
        handled_ele_str = deal_method(handled_ele_str)
        handled_ele_str = re.sub(pattern, "", handled_ele_str)
    return handled_ele_str


def get_ele_text(
        poco, selector_str, optional, deal_method_name, params_deal_module
):
    """
    get the text of the element, and after the text is obtained,
    you can choose whether to process it by a custom method
    """
    poco_ele.wait_exists(poco, selector_str, optional)
    poco_object = pm.create_poco_object_by_dsl(
        poco, selector_str, optional
    )
    ele_str = poco_object.get_text()
    if ele_str is None:
        ele_str = poco_object.attr('label')
    ele_str = ele_str.replace(u"\u200b", "")
    if not (deal_method_name is None):
        deal_method = getattr(params_deal_module, deal_method_name)
        ele_str = deal_method(ele_str)
    return ele_str


def text_change(poco, select_str, optional, o_text):
    """
    determine whether the copy of the element has changed within the specified
    time.
    """
    result = False
    timeout = optional["timeout"]
    current_wait_second = 1
    while (not result) and (timeout > 0):
        try:
            poco_target = pm.create_poco_object_by_dsl(
                poco, select_str, optional
            )
            if poco_target.exists():
                t_text = poco_target.get_text()
                if t_text is None:
                    t_text = poco_target.attr('label')
                if not (o_text == t_text):
                    result = True
            if result:
                break
        except Exception:
            pass
        time.sleep(current_wait_second)
        timeout -= current_wait_second
        current_wait_second += 1
    return result



# -*- coding: utf-8 -*-
"""
ios step implements class
"""

from flybirds.core.plugin.plugins.default.app_base_step import AppBaseStep

__open__ = ["Step"]


class Step(AppBaseStep):
    """IOS Step Class"""

    name = "ios_step"



# -*- coding: utf-8 -*-
"""
behave cannot get language from json report,use this to change the behave func
"""

import six
from behave.formatter.json import JSONFormatter
from behave.step_registry import StepRegistry
from flybirds.core.extend.step import load_steps

import flybirds.utils.flybirds_log as log
import flybirds.utils.language_helper as lge

# hold behave add step func
step_registry_add_step = StepRegistry.add_step_definition
language_list = lge.get_language_list()


# inject behave function: json format
def inject_behave():
    """
    change behave func
    """
    log.info("change behave json format feature to flybirds feature")
    JSONFormatter.feature = feature_wreap
    log.info(
        "change behave add_step_definition to flybirds add_step_definition"
    )
    if (StepRegistry.add_step_definition.__module__ == "behave.step_registry"
            or StepRegistry.add_step_definition.__name__
                    .find("add_step_definition") >= 0):
        StepRegistry.add_step_definition = step_registry_add_step_wreap


def feature_wreap(self, feature):
    """
    wreap behave feature
    """
    self.reset()
    self.current_feature = feature
    self.current_feature_data = {
        "keyword": feature.keyword,
        "name": feature.name,
        "tags": list(feature.tags),
        "location": six.text_type(feature.location),
        "status": None,  # Not known before feature run.
        "language": feature.language,
    }
    element = self.current_feature_data
    if feature.description:
        element["description"] = feature.description


def step_registry_add_step_wreap(self, keyword, step_text, func):
    """
    wreap behave add_step_definition
    """
    step_type = keyword.lower()

    if (
            step_type == "step"
            and language_list is not None
            and len(language_list) > 0
    ):
        for language_item in language_list:
            txt_list = lge.parse_glb_step(step_text, language_item)
            if txt_list is not None and len(txt_list) > 0:
                for txt in txt_list:
                    step_registry_add_step(self, keyword, txt, func)
    step_registry_add_step(self, keyword, step_text, func)


# start inject
inject_behave()

# load step
load_steps()



# -*- coding: utf-8 -*-
"""
This module is used to create cli project.
"""
import os
import platform
import shutil

import typer

import flybirds.template as template
import flybirds.utils.flybirds_log as log
from flybirds.utils.file_helper import get_files_from_dir, \
    get_paths_from_dir, \
    replace_file_content, update
from flybirds.utils.pkg_helper import find_package_base_path


def create_demo():
    """
    Create project cli demo
    """
    typer.secho(
        "Welcome to flybirds cli. Please enter any information to continue.",
        fg=typer.colors.MAGENTA,
    )
    project_name = typer.prompt("Please input your project name>>")
    test_platform = typer.prompt(
        "Please input your test platform?(Android/IOS)"
    )
    device_id = "127.0.0.1:8200"
    package_name = "ctrip.android.view"
    web_driver_agent = "com.fd.test.WebDriverAgentLib.xctrunner"
    if test_platform is not None and test_platform.strip().lower() == 'ios':
        package_name = "com.ctrip.inner.wireless"

        is_bundle = typer.confirm(
            "Do you want to configure your webDriverAgent now?"
            "(this step can be skipped)")
        if is_bundle:
            web_driver_agent = typer.prompt("Please input your Bundle ID of"
                                            " webDriverAgent?")
        else:
            typer.secho(
                "You can configure your  Bundle ID of webDriverAgent later in"
                " the project's"
                " flybirds_config.json file.", fg=typer.colors.YELLOW)

    connect_device = typer.confirm(
        "Do you want to configure your deviceId now?"
        "(this step can be skipped)")
    if connect_device:
        device_id = typer.prompt("Please input your deviceId?")
    else:
        typer.secho("You can configure your deviceId later in the project's"
                    " flybirds_config.json file.", fg=typer.colors.YELLOW)

    if_package = typer.confirm(
        "Do you want to configure your packageName now?"
        "(this step can be skipped)")
    if if_package:
        package_name = typer.prompt(
            "Please input your packageName?(You can use"
            " the ADB command to view your package name"
            ", such as: adb shell pm list packages |"
            " findstr 'trip')"
        )
    else:
        typer.secho("You can configure your packageName later in the project's"
                    " flybirds_config.json file.", fg=typer.colors.YELLOW)

    try:
        typer.echo(f"Cloning into '{project_name}'...")
        total = 700
        with typer.progressbar(length=total, label="Processing") as progress:
            demo_path = copy_from_template(progress, project_name,
                                           test_platform, device_id,
                                           package_name, web_driver_agent)
        typer.secho(
            f"Done it! Create Project {project_name} has success!\n"
            f"You can find it at: {demo_path}",
            fg=typer.colors.MAGENTA,
        )
    except Exception as e:
        typer.secho(
            f"Error!! create project {project_name} has error, errMsg: {e}",
            fg=typer.colors.MAGENTA,
            err=True,
        )


def copy_from_template(progress, project_name, test_platform, device_id=None,
                       package_name=None, web_driver_agent=None):
    """
    Generate project files from template
    """
    # Serialization path
    src_file_path = template.__file__
    src_path = os.path.normpath(src_file_path[0: src_file_path.rfind(os.sep)])
    target_path = os.path.normpath(
        os.path.join(os.path.normpath(os.getcwd()), project_name)
    )

    if os.path.isdir(target_path):
        # target_path is existed
        shutil.rmtree(target_path)
    shutil.copytree(src_path, target_path)
    progress.update(100)

    try:
        # process extend pkg
        add_extend_pkg(target_path)
    except Exception as e:
        log.error(f"[create_project][add_extend_pkg] has error, msg: {e}")

    progress.update(100)

    # delete file
    os.remove(os.path.normpath(os.path.join(target_path, "__init__.py")))
    progress.update(100)

    # modify platform
    if test_platform is not None:
        replace_file_content(
            os.path.join(target_path, "config/flybirds_config.json"),
            "platform",
            test_platform,
        )
    progress.update(100)

    # modify deviceId
    if device_id is not None:
        replace_file_content(
            os.path.join(target_path, "config/flybirds_config.json"),
            "deviceId",
            device_id,
        )
    progress.update(100)

    # modify packageName
    if package_name is not None:
        replace_file_content(
            os.path.join(target_path, "config/flybirds_config.json"),
            "packageName",
            package_name,
        )
    progress.update(100)

    # modify webDriverAgent
    if web_driver_agent is not None:
        replace_file_content(
            os.path.join(target_path, "config/flybirds_config.json"),
            "webDriverAgent",
            web_driver_agent,
        )
    progress.update(100)
    return target_path


def run_project(progress, target_path):
    """
    install packages
    """
    target_venv_path = os.path.normpath(
        os.path.join(target_path, activate_venv())
    )

    os.system(
        f"cd {target_path} && virtualenv venv && {target_venv_path} "
        f"&& pip install -r requirements.txt"
    )
    progress.update(100)


def activate_venv():
    """
    Activate venv
    :return:
    """
    sys_str = platform.system()
    if sys_str == "Windows":
        return "./venv/Scripts/activate"
    else:
        return "source venv/bin/activate"


def add_extend_pkg(demo_path):
    """
    Add expansion pack
    """
    pkg_query = "-flybirds-plugin"
    pkg_list = find_package_base_path(pkg_query)
    if pkg_list is None or len(pkg_list) <= 0:
        log.info("[create_project][add_extend_pkg] has no extend packs need to"
                 "be added.")
        return

    copy_extend_files(pkg_list, demo_path)


def copy_extend_files(pkg_list, demo_pro_path):
    """
    Add extend features and config files
    """
    if demo_pro_path is None:
        return

    for pkg in pkg_list:
        # features src path
        extend_path = os.path.normpath(
            os.path.join(pkg.get("path"), pkg.get("name"), 'template'))

        if extend_path is None or not os.path.exists(extend_path):
            log.info(
                "[create_project][copy_extend_files]extend_path is none or not"
                "existed.")
            continue

        feat_path = os.path.join(os.path.normpath(extend_path), 'features')
        config_path = os.path.join(os.path.normpath(extend_path), 'config')

        # add extend features
        if feat_path is not None and os.path.exists(feat_path):
            feat_files = get_files_from_dir(feat_path)

            # features target path
            demo_an_paths = get_paths_from_dir(demo_pro_path, 'android')
            demo_ios_paths = get_paths_from_dir(demo_pro_path, 'ios')

            for an_path in demo_an_paths:
                for file in feat_files:
                    shutil.copy(file, an_path)

            for ios_path in demo_ios_paths:
                for file in feat_files:
                    shutil.copy(file, ios_path)

        # add extend config
        if config_path is not None and os.path.exists(config_path):
            # config target path
            demo_config_path = os.path.join(os.path.normpath(demo_pro_path),
                                            'config')
            if os.path.isdir(demo_config_path):
                # target_path is existed
                shutil.rmtree(demo_config_path)
            shutil.copytree(config_path, demo_config_path)


def write_import_steps(pkg_list, demo_pro_path, site_path):
    """
    Write the steps that needs to be imported
    """
    if site_path is None:
        return

    import_str = ''
    # str that need to be imported
    for pkg in pkg_list:
        step_path = os.path.normpath(
            os.path.join(pkg.get("path"), pkg.get("name"), 'dsl', 'step'))

        if step_path is None or not os.path.exists(step_path):
            log.info(
                "[create_project][write_import_steps] extend_step path is none"
                "or not existed.")
            continue

        step_files = os.listdir(step_path)
        pkg_import_str = ''
        if step_files is not None and len(step_files) > 0:
            pkg_import_str = f'from {pkg.get("name")}.dsl.step import'
        for file in step_files:
            stem, suffix = os.path.splitext(file)
            if '__init__' == stem or '__pycache__' == stem:
                continue
            pkg_import_str += " " + stem + ","

        import_str += pkg_import_str.strip(',') + "\n"

    # write the extension steps that need to be imported into the file
    steps_file = os.path.join(os.path.normpath(demo_pro_path),
                              'features/steps/steps.py')
    update(steps_file, import_str)



# -*- coding: utf-8 -*-
"""
ui driver proxy
"""
from flybirds.core.global_context import GlobalContext


def poco_init():
    return GlobalContext.ui_driver.poco_init()


def air_bdd_screen_size(dr_instance):
    return GlobalContext.ui_driver.air_bdd_screen_size(dr_instance)



# -*- coding: utf-8 -*-
"""
it is triggered when behave before all
"""
import base64
import traceback
from flybirds.core.config_manage import ConfigManage, AppEnvConfig
from flybirds.core.script_config import ScriptImportManage
from flybirds.report.fail_feature_create import set_rerun_info
import flybirds.core.global_resource as gr
import flybirds.utils.flybirds_log as log
from flybirds.core.global_context import GlobalContext


class OnConfigLoad:  # pylint: disable=too-few-public-methods
    """
    load user config data event
    """

    name = "OnConfigLoad"
    order = 0

    @staticmethod
    def can(context):
        return True

    @staticmethod
    def run(context):
        """
        execute load config and set to global context
        """
        try:
            log.info("start on before hook")
            # initialize the global object module
            gr.init_glb()
            # get user-specified parameter value or custom parameter
            user_data = (
                context.config.userdata
                if isinstance(context.config.userdata, dict)
                else {}
            )
            for key, value in user_data.items():
                user_data[key] = str(base64.b64decode(value), "utf-8")
            gr.set_value("userData", user_data)
            # obtain the failure case information and associate
            # it with the report later
            set_rerun_info(user_data, gr)
            # get configuration, user-defined priority and configuration file
            config_manage = ConfigManage(user_data)
            gr.set_value("configManage", config_manage)
            context.config_manage = config_manage
            log.info("configuration file read completed")
            log.ch.setLevel(gr.get_log_level())

            # config app runtime env
            app_env_config = AppEnvConfig(user_data, None)
            if app_env_config.env_config is not None:
                gr.set_value("appEnvConfig", app_env_config.env_config)
                context.app_env_config = app_env_config
                log.info(
                    "app operating environment variable "
                    "configuration is complete"
                )

            gr.set_value("run_info", config_manage.run_config)

            # get the custom python script of the project
            project_script = ScriptImportManage()
            gr.set_value("projectScript", project_script)
            context.project_script = project_script
            log.info("the python script in the project is read")

        except Exception as e_out:
            log.info("global initialization error", traceback.format_exc())
            raise e_out


var = GlobalContext.insert("config_processor", OnConfigLoad, 1)



# -*- coding: utf-8 -*-
from flybirds.core.global_context import GlobalContext as g_Context
from flybirds.utils import language_helper as lan


def get_swipe_search_start_point(direction, start_x=None, start_y=None):
    """
    Get the start coordinate point of the sliding operation
    """
    # get current language
    language = g_Context.get_current_language()
    direct_left = lan.parse_glb_str("left", language)
    direct_right = lan.parse_glb_str("right", language)
    direct_up = lan.parse_glb_str("up", language)
    direct_down = lan.parse_glb_str("down", language)

    start_point = [None, None]
    if start_x is None:
        if direction == direct_left:
            start_point[0] = 0.666
        elif direction == direct_right:
            start_point[0] = 0.333
        else:
            start_point[0] = 0.5
    else:
        start_point[0] = start_x
    if start_y is None:
        if direction == direct_up:
            start_point[1] = 0.666
        elif direction == direct_down:
            start_point[1] = 0.333
        else:
            start_point[1] = 0.5
    else:
        start_point[1] = start_y
    return start_point


def search_direction_switch(direction):
    """
    To find in a certain direction,
    you have to slide in a certain direction
    to change the direction parameter
    """
    # get current language
    language = g_Context.get_current_language()
    direct_left = lan.parse_glb_str("left", language)
    direct_right = lan.parse_glb_str("right", language)
    direct_up = lan.parse_glb_str("up", language)
    direct_down = lan.parse_glb_str("down", language)

    direction = direction.strip()
    if direction == direct_left:
        direction = direct_right
    elif direction == direct_right:
        direction = direct_left
    elif direction == direct_up:
        direction = direct_down
    elif direction == direct_down:
        direction = direct_up
    return direction



# -*- coding: utf-8 -*-
"""
ios screen imp
"""
from flybirds.core.plugin.plugins.default import screen

__open__ = ["Screen"]


class Screen:
    """
    screen imp
    """
    name = "ios_screen"

    def screen_shot(self, path):
        screen.screen_shot(path)

    def screen_link_to_behave(self, scenario, step_index, tag=None):
        screen.screen_link_to_behave(scenario, step_index, tag)



# -*- coding: utf-8 -*-
"""
Step implement of element click.
"""
import flybirds.core.global_resource as gr
import flybirds.core.plugin.plugins.default.ui_driver.poco.findsnap as find_snap
import flybirds.utils.dsl_helper as dsl_helper
from flybirds.core.global_context import GlobalContext as g_Context


def click_ele(context, param):
    """
    Click  element
    """
    param_dict = dsl_helper.params_to_dic(param)
    poco_instance = gr.get_value("pocoInstance")

    selector_str = param_dict["selector"]
    optional = {}
    if "path" in param_dict.keys():
        optional["path"] = param_dict["path"]
    elif "multiSelector" in param_dict.keys():
        optional["multiSelector"] = param_dict["multiSelector"]
    if "timeout" in param_dict.keys():
        optional["timeout"] = float(param_dict["timeout"])
    else:
        optional["timeout"] = gr.get_frame_config_value("wait_ele_timeout", 10)

    verify_dsl_str = None
    verify_optional = {}
    verify_action = None
    if "verifyEle" in param_dict.keys():
        verify_dsl_str = param_dict["verifyEle"]
        verify_action = param_dict["verifyAction"]
    if "verifyIsPath" in param_dict.keys():
        verify_optional["path"] = param_dict["verifyIsPath"]
    elif "verifyIsMultiSelector" in param_dict.keys():
        verify_optional["multiSelector"] = param_dict["verifyIsMultiSelector"]
    if "verifyTimeout" in param_dict.keys():
        verify_optional["timeout"] = float(param_dict["verifyTimeout"])
    else:
        verify_optional["timeout"] = gr.get_frame_config_value(
            "click_verify_timeout", 10
        )

    g_Context.element.air_bdd_click(
        poco_instance,
        selector_str,
        optional,
        verify_dsl_str,
        verify_optional,
        verify_action,
    )


def click_text(context, param):
    param_dict = dsl_helper.params_to_dic(param)
    poco_instance = gr.get_value("pocoInstance")

    selector_str = param_dict["selector"]
    if "fuzzyMatch" in param_dict.keys():
        selector_str = "textMatches=" + selector_str
    else:
        selector_str = "text=" + selector_str
    optional = {}
    if "timeout" in param_dict.keys():
        optional["timeout"] = float(param_dict["timeout"])
    else:
        optional["timeout"] = gr.get_frame_config_value("wait_ele_timeout", 10)

    verify_dsl_str = None
    verify_optional = {}
    verify_action = None
    if "verifyEle" in param_dict.keys():
        verify_dsl_str = param_dict["verifyEle"]
        verify_action = param_dict["verifyAction"]
    if "verifyIsPath" in param_dict.keys():
        verify_optional["path"] = param_dict["verifyIsPath"]
    elif "verifyIsMultiSelector" in param_dict.keys():
        verify_optional["multiSelector"] = param_dict["verifyIsMultiSelector"]
    if "verifyTimeout" in param_dict.keys():
        verify_optional["timeout"] = float(param_dict["verifyTimeout"])
    else:
        verify_optional["timeout"] = gr.get_frame_config_value(
            "click_verify_timeout", 10
        )

    g_Context.element.air_bdd_click(
        poco_instance,
        selector_str,
        optional,
        verify_dsl_str,
        verify_optional,
        verify_action,
    )


def click_coordinates(context, x, y):
    poco_instance = gr.get_value("pocoInstance")
    screen_size = gr.get_device_size()
    x_coordinate = float(x) / screen_size[0]
    y_coordinate = float(y) / screen_size[1]
    poco_instance.click([x_coordinate, y_coordinate])
    if gr.get_frame_config_value("use_snap", False):
        find_snap.fix_refresh_status(True)



# -*- coding: utf-8 -*-
"""
This module defines the steps related to the UI element.
"""
from behave import step

from flybirds.core.global_context import GlobalContext as g_Context
from flybirds.utils.dsl_helper import get_params


@step("text[{param1}]property[{param2}]is {param3}")
def text_attr_equal(context, param1=None, param2=None, param3=None):
    param_1, param_2, param_3 = get_params(
        context, (param1, "param1"), (param2, "param2"), (param3, "param3")
    )
    g_Context.step.text_attr_equal(context, param_1, param_2, param_3)


@step("element[{param1}]property[{param2}]is {param3}")
def ele_attr_equal(context, param1=None, param2=None, param3=None):
    param_1, param_2, param_3 = get_params(
        context, (param1, "param1"), (param2, "param2"), (param3, "param3")
    )
    g_Context.step.ele_attr_equal(context, param_1, param_2, param_3)


@step("click[{param}]")
def click_ele(context, param=None):
    (param_1,) = get_params(context, (param, "param"))
    g_Context.step.click_ele(context, param_1)


@step("click text[{param}]")
def click_text(context, param=None):
    (param_1,) = get_params(context, (param, "param"))
    g_Context.step.click_text(context, param_1)


@step("click position[{x},{y}]")
def click_coordinates(context, x=None, y=None):
    param_x, param_y = get_params(context, (x, "param1"), (y, "param2"))
    g_Context.step.click_coordinates(context, param_x, param_y)


@step("in[{param1}]input[{param2}]")
def ele_input(context, param1=None, param2=None):
    param_1, param_2 = get_params(
        context, (param1, "param1"), (param2, "param2")
    )
    g_Context.step.ele_input(context, param_1, param_2)


@step("element[{param1}]position not change in[{param2}]seconds")
def position_not_change(context, param1=None, param2=None):
    param_1, param_2 = get_params(
        context, (param1, "param1"), (param2, "param2")
    )
    g_Context.step.position_not_change(context, param_1, param_2)


@step("[{param1}]slide to {param2} distance[{param3}]")
def ele_swipe(context, param1=None, param2=None, param3=None):
    param_1, param_2, param_3 = get_params(
        context, (param1, "param1"), (param2, "param2"), (param3, "param3")
    )
    g_Context.step.ele_swipe(context, param_1, param_2, param_3)


@step("slide to {param1} distance[{param2}]")
def full_screen_swipe(context, param1=None, param2=None):
    param_1, param_2 = get_params(
        context, (param1, "param1"), (param2, "param2")
    )
    g_Context.step.full_screen_swipe(context, param_1, param_2)


@step("exist text[{param}]")
def wait_text_exist(context, param=None):
    (param_1,) = get_params(context, (param, "param"))
    g_Context.step.wait_text_exist(context, param_1)


@step("not exist text[{param}]")
def text_not_exist(context, param=None):
    (param_1,) = get_params(context, (param, "param"))
    g_Context.step.text_not_exist(context, param_1)


@step("text[{param}]disappear")
def wait_text_disappear(context, param=None):
    (param_1,) = get_params(context, (param, "param"))
    g_Context.step.wait_text_disappear(context, param_1)


@step("exist[{param}]element")
def wait_ele_exit(context, param=None):
    (param_1,) = get_params(context, (param, "param"))
    g_Context.step.wait_ele_exit(context, param_1)


@step("not exist element[{param}]")
def ele_not_exit(context, param=None):
    (param_1,) = get_params(context, (param, "param"))
    g_Context.step.ele_not_exit(context, param_1)


@step("element[{param}]disappear")
def wait_ele_disappear(context, param=None):
    (param_1,) = get_params(context, (param, "param"))
    g_Context.step.wait_ele_disappear(context, param_1)


@step("the text of element[{param1}]is[{param2}]")
def ele_text_equal(context, param1=None, param2=None):
    param_1, param_2 = get_params(
        context, (param1, "param1"), (param2, "param2")
    )
    g_Context.step.ele_text_equal(context, param_1, param_2)


@step("the text of element[{param1}]include[{param2}]")
def ele_text_container(context, param1=None, param2=None):
    param_1, param_2 = get_params(
        context, (param1, "param1"), (param2, "param2")
    )
    g_Context.step.ele_text_container(context, param_1, param_2)


@step("page rendering complete appears element[{param}]")
def wait_ele_appear(context, param=None):
    (param_1,) = get_params(context, (param, "param"))
    g_Context.step.wait_ele_appear(context, param_1)


@step("existing element[{param}]")
def exist_ele(context, param=None):
    (param_1,) = get_params(context, (param, "param"))
    g_Context.step.exist_ele(context, param_1)


@step("in[{param1}]from {param2} find[{param3}]element")
def swipe_to_ele(context, param1=None, param2=None, param3=None):
    param_1, param_2, param_3 = get_params(
        context, (param1, "param1"), (param2, "param2"), (param3, "param3")
    )
    g_Context.step.swipe_to_ele(context, param_1, param_2, param_3)


@step("from {param1} find[{param2}]element")
def full_screen_swipe_to_ele_aaa(context, param1=None, param2=None):
    param_1, param_2 = get_params(
        context, (param1, "param1"), (param2, "param2")
    )
    g_Context.step.full_screen_swipe_to_ele_aaa(context, param_1, param_2)



# -*- coding: utf-8 -*-
"""
py ns load
"""
import importlib
import os
import pkgutil
import pkg_resources


def load_pkg_by_ns(pkg_ns):
    """
    py ns load
    """
    base = importlib.import_module(pkg_ns)
    for loader, module_name, is_pkg in \
            pkgutil.walk_packages(base.__path__, f'{base.__name__}.'):
        try:
            __import__(module_name)
        except ImportError as e:
            raise e


def find_package(pkg_query):
    if pkg_query is not None and pkg_query != "":
        working_set = pkg_resources.WorkingSet()
        pkg_list = []
        lst = [d for d in working_set]
        for item in lst:
            if pkg_query in item.project_name:
                pkg_list.append(item)
        if pkg_list is not None and len(pkg_list) > 0:
            pkg_root_list = []
            for item in pkg_list:
                pkg_root_list.append(
                    item.project_name.replace("-", "_"))
            return pkg_root_list
        else:
            return None
    else:
        return None


def find_package_base_path(pkg_query):
    if pkg_query is not None and pkg_query != "":
        working_set = pkg_resources.WorkingSet()
        pkg_list = []
        lst = [d for d in working_set]
        for item in lst:
            if pkg_query in item.project_name:
                pkg_list.append(item)
        if pkg_list is not None and len(pkg_list) > 0:
            pkg_root_list = []
            for item in pkg_list:
                if os.path.isfile(item.module_path):
                    new_item = {
                        "name": item.project_name.replace("-", "_"),
                        "path": os.path.dirname(item.module_path)
                    }
                else:
                    new_item = {
                        "name": item.project_name.replace("-", "_"),
                        "path": item.module_path
                    }

                pkg_root_list.append(new_item)
            return pkg_root_list
        else:
            return None
    else:
        return None



# -*- coding: utf-8 -*-
"""
equal includ helper
"""
from flybirds.core.exceptions import FlybirdVerifyException


def text_equal(o_text, t_text):
    """
    Determine whether the text is equal
    """
    if o_text != t_text:
        message = "text not equal, expect value:{}, actual value:{}".format(
            o_text, t_text
        )
        raise FlybirdVerifyException(message)


def text_container(o_text, t_text):
    """
    Determine whether the text is included
    """
    if not (o_text in t_text):
        message = "text not contain, expect value include:{}," \
                  " actual value:{}".format(o_text, t_text)
        raise FlybirdVerifyException(message)


def attr_equal(o_attr, t_attr):
    """
    Determine whether the attributes are equal
    """
    if str(o_attr) != str(t_attr):
        message = "attr not equal, expect value:{}, actual value:{}".format(
            o_attr, t_attr
        )
        raise FlybirdVerifyException(message)



# -*- coding: utf-8 -*-
"""
ios app core api implement
"""

from airtest.core.api import (time, start_app, stop_app)

__open__ = ["App"]


class App:
    """IOS App Class"""

    name = "ios_app"

    def wake_app(self, package_name, wait_time=None):
        """
        Start the target application on device
        """
        start_app(package_name)
        if not (wait_time is None):
            time.sleep(wait_time)

    def shut_app(self, package_name):
        """
        stop the target application on device
        """
        stop_app(package_name)

    def install_app(self, package_path, wait_time=None):
        raise NotImplementedError

    def uninstall_app(self, package_name, wait_time=None):
        raise NotImplementedError



# -*- coding: utf-8 -*-
"""
Parameter process
"""


def replace_comma(u_text):
    """
    Replace the English comma in the text, because the English comma
     has special meaning in the FLYBIRDS framework
    """
    return u_text.replace(',', ' ')



# -*- coding: utf-8 -*-
"""
flybirds common error
"""


class FlybirdNotFoundException(Exception):
    """
    not find flybird
    """

    def __init__(self, message, select_dic, error=None):
        message = f"selectors={str(select_dic)} {message}"
        if error is not None:
            message = f"{message} innerErr:{error}"
        super().__init__(message)
        self.message = message

    def __str__(self):
        return str(self.message)


class PositionNotChangeException(Exception):
    """
    position not change
    """

    def __init__(self, message):
        super().__init__(message)
        self.message = message

    def __str__(self):
        return str(self.message)


class FlybirdCallMethodParamsException(Exception):
    """
    params error
    """

    def __init__(self, method, param_name):
        message = f"call method:{method} has invalid params:{param_name}"
        super().__init__(message)
        self.message = message

    def __str__(self):
        return str(self.message)


class FlybirdEleExistsException(Exception):
    """
    ele not exists
    """

    def __init__(self, message):
        super().__init__(message)
        self.message = message

    def __str__(self):
        return str(self.message)


class FlybirdVerifyException(Exception):
    """
    verify error
    """

    def __init__(self, message):
        super().__init__()
        self.message = message

    def __str__(self):
        return str(self.message)


class FlybirdPositionChanging(Exception):
    """
    position changing
    """

    def __init__(self, message):
        super().__init__()
        self.message = message

    def __str__(self):
        return str(self.message)


class ScreenRecordException(Exception):
    """
    screen record error
    """

    def __init__(self, message):
        super().__init__()
        self.message = message

    def __str__(self):
        return str(self.message)



# -*- coding: utf-8 -*-
"""
This module contains the UI elements Core APIs.
"""

from flybirds.core.global_context import GlobalContext as g_context

"""
UI elements Operations
"""


def str_input(input_str, after_input_wait=None):
    """
    Input text on the target device. Text input widget must be active first.
    :param input_str: text to input, unicode is supported
    :param after_input_wait: waiting time after input
    :return:
    """
    g_context.element.str_input(input_str, after_input_wait=after_input_wait)


def key_event(key_code):
    """
      Perform key event on the device.
    * ``iOS``: Only supports home/volumeUp/volumeDown::
    :param key_code:  platform specific key name
    :return: None
    """
    g_context.element.key_event(key_code)


"""
poco UI object Operations
"""


def ui_driver_init():
    """init ui driver"""
    return g_context.element.ui_driver_init()


def get_ele_attr(
    poco,
    selector_str,
    optional,
    attr_name,
    deal_method=None,
    params_deal_module=None,
):
    """
    Get the specified attribute of the element, and after the attribute is
    obtained, you can choose whether to process it through a custom method.
    """
    return g_context.element.get_ele_attr(
        poco,
        selector_str,
        optional,
        attr_name,
        deal_method=deal_method,
        params_deal_module=params_deal_module,
    )


def air_bdd_click(
    poco,
    select_dsl_str,
    optional,
    verify_dsl_str=None,
    verify_optional=None,
    verify_action=None,
):
    """
    Click the element. optional parameters to determine whether the clicked
    page is rendered.
    """
    g_context.element.air_bdd_click(
        poco,
        select_dsl_str,
        optional,
        verify_dsl_str=verify_dsl_str,
        verify_optional=verify_optional,
        verify_action=verify_action,
    )


def verify_click_end(
    poco, verify_dsl_str, verify_optional, verify_action, o_position, o_text
):
    """
    verify_click_end
    """
    g_context.element.verify_click_end(
        poco,
        verify_dsl_str,
        verify_optional,
        verify_action,
        o_position,
        o_text,
    )


def wait_exists(poco, selector_str, optional):
    """
     determine whether the element exists within the optional
    :param poco: ui driver
    :param selector_str: the string of selector
    :param optional:
    :return: None
    """
    g_context.element.wait_exists(poco, selector_str, optional)


def not_exist(poco, selector_str, optional):
    """
    determine whether the element does not exist
    :param poco: ui driver
    :param selector_str: the string of selector
    :param optional:
    :return: None
    """
    g_context.element.not_exist(poco, selector_str, optional)


def wait_disappear(poco, selector_str, optional):
    """
    determine whether the element disappears within the optional
    :param poco: ui driver
    :param selector_str:
    :param optional:
    :return: None
    """
    g_context.element.wait_disappear(poco, selector_str, optional)


def detect_error():
    """
    detect errors
    """
    g_context.element.detect_error()


def find_ele_by_snap(poco, config, optional):
    """
    Determine whether the element exists in the snapshot within optional
    :param poco: ui driver
    :param config:
    :param optional:
    :return: None
    """
    g_context.element.find_ele_by_snap(poco, config, optional)


def verify_ele_by_snap(poco, config, optional):
    """
     use snapshots to judge element text
    :param poco: ui driver
    :param config:
    :param optional:
    :return:None
    """
    g_context.element.verify_ele_by_snap(poco, config, optional)


def air_bdd_input(
    poco, select_dsl_str, optional, input_str, after_input_wait=None
):
    """
    Input text
    :return:
    """
    g_context.element.air_bdd_input(
        poco,
        select_dsl_str,
        optional,
        input_str,
        after_input_wait=after_input_wait,
    )


def create_poco_object_by_dsl(poco, select_dsl_str, optional):
    """
    get the poco element of optional parameter structure and selector structure
    :param poco:
    :param select_dsl_str:
    :param optional:
    :return: ui driver instance
    """
    return g_context.element.create_poco_object_by_dsl(
        poco, select_dsl_str, optional
    )


def position_change(poco, select_str, optional, o_position):
    """
    determine whether the position of the element has changed within the
    specified time
    :return: 'False' or 'True'
    """
    return g_context.element.position_change(
        poco, select_str, optional, o_position
    )


def position_not_change(poco, select_str, optional, dur_time, verify_count):
    """
    determine the position of the element has not changed
    """
    g_context.element.position_not_change(
        poco, select_str, optional, dur_time, verify_count
    )


def air_bdd_screen_size(poco_instance):
    """
    get the screen size of the device in use
    :param poco_instance: ui  driver
    :return: tuple: float number indicating the screen physical resolution
    in pixels
    """
    return g_context.element.air_bdd_screen_size(poco_instance)


def create_poco_object(poco, select_dic={}):
    """
    create page elements corresponding to pocoUi elements
    :param poco:
    :param select_dic: Constraints for constructing pocoObject
    :return:
    """
    return g_context.element.create_poco_object(poco, select_dic)


def create_parent(poco_object):
    """
    create the parent element of the pocoUi element
    :param poco_object:
    :return:
    """
    return g_context.element.create_parent(poco_object)


def create_first_child(poco_object, select_dic={}):
    """
    create the first child element of the pocoUi element
    :param poco_object:
    :param select_dic:
    :return:
    """
    return g_context.element.create_first_child(
        poco_object, select_dic=select_dic
    )


def create_first_offspring(poco_object, select_dic={}):
    """
    create the first descendant element of the pocoUi element
    :param poco_object:
    :param select_dic:
    :return:
    """
    return g_context.element.create_first_offspring(
        poco_object, select_dic=select_dic
    )


def create_first_sibling(poco_object, select_dic={}):
    """
    create the first sibling element of the pocoUi element
    :param poco_object:
    :param select_dic:
    :return:
    """
    return g_context.element.create_first_sibling(
        poco_object, select_dic=select_dic
    )


def select_child(poco_object, target_index, select_dic={}):
    """
    select the child element of the pocoUi element
    if pocoUi does not exist on the page or the pocoUi element does not have
    any child elements or the number of child elements is less than
    target_index,an exception is thrown.
    :param poco_object:
    :param target_index:
    :param select_dic:
    :return:
    """
    return g_context.element.select_child(
        poco_object, target_index, select_dic=select_dic
    )


def select_offspring(poco_object, target_index, select_dic={}):
    """
    select which descendant element of pocoUi element
    if pocoUi does not exist on the page or the pocoUi element does not have
    any descendant elements or the number of descendant elements is less than
    target_index, an exception is thrown.
    :param poco_object:
    :param target_index:
    :param select_dic:
    :return:
    """
    return g_context.element.select_offspring(
        poco_object, target_index, select_dic=select_dic
    )


def select_sibling(poco_object, target_index, select_dic={}):
    """
    select which sibling element of pocoUi element
    if pocoUi does not exist on the page or the pocoUi element does not have
    any sibling elements or the number of sibling elements is less than
    target_index, an exception is thrown.
    :param poco_object:
    :param target_index:
    :param select_dic:
    :return:
    """
    return g_context.element.select_sibling(
        poco_object, target_index, select_dic=select_dic
    )


def air_bdd_full_screen_swipe(
    poco,
    start_point,
    screen_size,
    direction,
    distance,
    duration,
    ready_time=None,
):
    """
    In the full screen range, slide from the specified starting point in one
    direction up, down, left, and right to specify the distance.
    """
    g_context.element.air_bdd_full_screen_swipe(
        poco,
        start_point,
        screen_size,
        direction,
        distance,
        duration,
        ready_time=ready_time,
    )


def air_bdd_ele_swipe(
    poco,
    container_dsl_str,
    optional,
    start_point,
    screen_size,
    direction,
    distance,
    duration,
    ready_time=None,
):
    """
    Slides a specified distance up, down, left, or right from a specified
    starting point within a sliding element.
    """
    g_context.element.air_bdd_ele_swipe(
        poco,
        container_dsl_str,
        optional,
        start_point,
        screen_size,
        direction,
        distance,
        duration,
        ready_time=ready_time,
    )


def air_bdd_direction_swipe(
    poco, start_point, direction, distance, duration=None
):
    """
    swipe the specified distance from the starting point to one of up, down,
    left, and right.
    :param poco:
    :param start_point:
    :param direction:
    :param distance:
    :param duration:
    :return:
    """
    g_context.element.air_bdd_direction_swipe(
        poco, start_point, direction, distance, duration=duration
    )


def air_bdd_percent_point_swipe(poco, start_point, end_point, duration=None):
    """
    swipe from the start point to the end point
    :param poco:
    :param start_point:
    :param end_point:
    :param duration:
    :return:
    """
    g_context.element.air_bdd_percent_point_swipe(
        poco, start_point, end_point, duration=duration
    )


def full_screen_swipe_search(
    poco,
    search_dsl_str,
    search_optional,
    swipe_count,
    direction,
    screen_size,
    start_x=None,
    start_y=None,
    distance=None,
    duration=None,
):
    """
    Full screen swipe to find
    """
    g_context.element.full_screen_swipe_search(
        poco,
        search_dsl_str,
        search_optional,
        swipe_count,
        direction,
        screen_size,
        start_x=start_x,
        start_y=start_y,
        distance=distance,
        duration=duration,
    )


def air_bdd_swipe_search(
    poco,
    container_dsl_str,
    container_optional,
    search_dsl_str,
    search_optional,
    swipe_count,
    screen_size,
    direction,
    start_x=None,
    start_y=None,
    distance=None,
    duration=None,
):
    """
    swipe in a certain direction in the full screen or within an element to
    find an element.
    """
    g_context.element.air_bdd_swipe_search(
        poco,
        container_dsl_str,
        container_optional,
        search_dsl_str,
        search_optional,
        swipe_count,
        screen_size,
        direction,
        start_x=start_x,
        start_y=start_y,
        distance=distance,
        duration=duration,
    )


def get_ele_text_replace_space(
    poco, selector_str, optional, deal_method_name, params_deal_module
):
    """
    get the text of the element, and after the text is obtained,
    you can choose whether to process it by a custom method
    """
    return g_context.element.get_ele_text_replace_space(
        poco, selector_str, optional, deal_method_name, params_deal_module
    )


def get_ele_text(
    poco, selector_str, optional, deal_method_name, params_deal_module
):
    """
    get the text of the element, and after the text is obtained,
    you can choose whether to process it by a custom method
    """
    return g_context.element.get_ele_text(
        poco, selector_str, optional, deal_method_name, params_deal_module
    )


def text_change(poco, select_str, optional, o_text):
    """
    determine whether the copy of the element has changed within the specified
    time.
    """
    return g_context.element.text_change(poco, select_str, optional, o_text)


def ele_text_is(
    poco,
    selector_str,
    target_str,
    optional,
    deal_method=None,
    params_deal_module=None,
):
    """
    determine whether the element is the expected value
    """
    g_context.element.ele_text_is(
        poco,
        selector_str,
        target_str,
        optional,
        deal_method=deal_method,
        params_deal_module=params_deal_module,
    )


def ele_text_contains(
    poco,
    selector_str,
    target_str,
    optional,
    deal_method=None,
    params_deal_module=None,
):
    """
    determine whether the element contains
    """
    g_context.element.ele_text_contains(
        poco,
        selector_str,
        target_str,
        optional,
        deal_method=deal_method,
        params_deal_module=params_deal_module,
    )


def ele_attr_is(
    poco,
    selector_str,
    optional,
    target_attr,
    target_attr_value,
    deal_method,
    params_deal_module,
):
    """
    determine whether the specified attribute of the element is the expected
    value.
    """
    g_context.element.ele_attr_is(
        poco,
        selector_str,
        optional,
        target_attr,
        target_attr_value,
        deal_method,
        params_deal_module,
    )



# -*- coding: utf-8 -*-
"""
Screen record.
"""
import flybirds.core.global_resource as gr


def start_screen_record_timeout(context, param):
    screen_record = gr.get_value("screenRecord")
    timeout = float(param)
    screen_record.start_record(timeout)


def start_screen_record(context):
    screen_record = gr.get_value("screenRecord")
    timeout = gr.get_frame_config_value("screen_record_time", 60)
    screen_record.start_record(timeout)


def stop_screen_record(context):
    screen_record = gr.get_value("screenRecord")
    screen_record.stop_record()
    step_index = context.cur_step_index - 1
    screen_record.link_record(context.scenario, step_index)



# -*- coding: utf-8 -*-
"""
Poco element apis
"""
import time

import flybirds.core.global_resource as gr
import flybirds.core.plugin.plugins.default.ui_driver.poco.findsnap as find_snap
import flybirds.core.plugin.plugins.default.ui_driver.poco.poco_manage as pm
import flybirds.utils.flybirds_log as log
from flybirds.core.exceptions import FlybirdEleExistsException
from flybirds.core.exceptions import FlybirdVerifyException
from flybirds.core.global_context import GlobalContext as g_Context
from flybirds.utils import language_helper as lan


def wait_exists(poco, selector_str, optional):
    """
    determine whether the element exists within the specified time
    """
    timeout = optional["timeout"]
    current_wait_second = 1
    find_success = False
    while timeout > 0:
        create_success = False
        try:
            poco_target = pm.create_poco_object_by_dsl(
                poco, selector_str, optional
            )
            create_success = True
            search_time = current_wait_second
            if search_time > 3:
                search_time = 3
            ele_exists = poco_target.exists()
            log.info(
                "wait_exists: {}, ele_exists: {}, timeout: {}".format(
                    selector_str, ele_exists, timeout
                )
            )

            if ele_exists:
                find_success = True
                break

            # modal error detection
            detect_error()

            poco_target.wait_for_appearance(timeout=search_time)
            find_success = True
            log.info(
                "wait_for_appearance: find_success: {}, timeout: {}".format(
                    find_success, timeout
                )
            )
            break
        except Exception:
            if not create_success:
                time.sleep(current_wait_second)
        if current_wait_second > 3:
            time.sleep(current_wait_second - 3)
        timeout -= current_wait_second
        current_wait_second += 2
    if not find_success:
        message = "during {}s time, not find {} in page".format(
            optional["timeout"], selector_str
        )
        raise FlybirdVerifyException(message)


def not_exist(poco, selector_str, optional):
    """
    determine whether the element does not exist
    """
    ele_exists = False
    try:
        poco_object = pm.create_poco_object_by_dsl(
            poco, selector_str, optional
        )
        ele_exists = poco_object.exists()
    except Exception:
        pass
    if ele_exists:
        message = "{} exists in page".format(selector_str)
        raise FlybirdEleExistsException(message)


def wait_disappear(poco, selector_str, optional):
    """
    determine whether the element disappears within the specified time
    """
    timeout = optional["timeout"]
    current_wait_second = 1
    disappear_success = False
    while timeout > 0:
        create_success = False
        try:
            poco_target = pm.create_poco_object_by_dsl(
                poco, selector_str, optional
            )
            create_success = True
            search_time = current_wait_second
            if search_time > 3:
                search_time = 3
            poco_target.wait_for_disappearance(timeout=search_time)
            disappear_success = True
            break
        except Exception:
            if not create_success:
                time.sleep(current_wait_second)
        if current_wait_second > 3:
            time.sleep(current_wait_second - 3)
        timeout -= current_wait_second
        current_wait_second += 1
    if not disappear_success:
        message = "during {}s time, {} not disappear in page".format(
            optional["timeout"], selector_str
        )
        raise FlybirdVerifyException(message)


def detect_error():
    language = g_Context.get_current_language()
    modal_list = lan.parse_glb_str("modal_list", language)
    poco = g_Context.ui_driver_instance

    for error_str in modal_list:
        error_target = pm.create_poco_object_by_dsl(
            poco, error_str, None
        )
        is_existed = error_target.exists()
        if is_existed:
            error_target.click()
            if gr.get_frame_config_value("use_snap", False):
                find_snap.fix_refresh_status(True)
            log.info("detect_error: {}, layer_errors_exists: true"
                     .format(error_str))
            time.sleep(0.5)



# -*- coding: utf-8 -*-
"""
Element verification
"""
import re

import flybirds.core.global_resource as gr
import flybirds.core.plugin.plugins.default.ui_driver.poco.poco_ele\
    as poco_ele
import flybirds.core.plugin.plugins.default.ui_driver.poco.poco_findsnap\
    as poco_find_snap
import flybirds.core.plugin.plugins.default.ui_driver.poco.poco_verify\
    as poco_verify
import flybirds.utils.dsl_helper as dsl_helper


def wait_text_exist(context, param):
    poco_instance = gr.get_value("pocoInstance")

    param_dict = dsl_helper.params_to_dic(param)
    selector_str = param_dict["selector"]
    config = None
    use_snap = gr.get_frame_config_value("use_snap", False)
    if "fuzzyMatch" in param_dict.keys():
        if use_snap:
            config = {"textMatches": selector_str}
        else:
            selector_str = "textMatches=" + selector_str
    else:
        if use_snap:
            config = {"text": selector_str}
        else:
            selector_str = "text=" + selector_str
    optional = {}
    if "timeout" in param_dict.keys():
        optional["timeout"] = float(param_dict["timeout"])
    else:
        optional["timeout"] = gr.get_frame_config_value("wait_ele_timeout", 10)
    if use_snap and config:
        poco_find_snap.find_ele_by_snap(poco_instance, config, optional)
    else:
        poco_ele.wait_exists(poco_instance, selector_str, optional)


def text_not_exist(context, param):
    poco_instance = gr.get_value("pocoInstance")

    param_dict = dsl_helper.params_to_dic(param)
    selector_str = param_dict["selector"]
    if "fuzzyMatch" in param_dict.keys():
        selector_str = "textMatches=" + selector_str
    else:
        selector_str = "text=" + selector_str
    optional = {}

    poco_ele.not_exist(poco_instance, selector_str, optional)


def wait_text_disappear(context, param):
    poco_instance = gr.get_value("pocoInstance")

    param_dict = dsl_helper.params_to_dic(param)
    selector_str = param_dict["selector"]
    if "fuzzyMatch" in param_dict.keys():
        selector_str = "textMatches=" + selector_str
    else:
        selector_str = "text=" + selector_str
    optional = {}
    if "timeout" in param_dict.keys():
        optional["timeout"] = float(param_dict["timeout"])
    else:
        optional["timeout"] = gr.get_frame_config_value(
            "wait_ele_disappear", 10
        )

    poco_ele.wait_disappear(poco_instance, selector_str, optional)


def wait_ele_exit(context, param):
    poco_instance = gr.get_value("pocoInstance")

    param_dict = dsl_helper.params_to_dic(param)
    selector_str = param_dict["selector"]
    optional = {}
    config = None
    use_snap = gr.get_frame_config_value("use_snap", False)
    if "path" in param_dict.keys():
        optional["path"] = param_dict["path"]
    elif "multiSelector" in param_dict.keys():
        optional["multiSelector"] = param_dict["multiSelector"]
    elif use_snap:
        config = {"name": selector_str}
    if "timeout" in param_dict.keys():
        optional["timeout"] = float(param_dict["timeout"])
    else:
        optional["timeout"] = gr.get_frame_config_value("wait_ele_timeout", 10)
    if use_snap and config:
        print("Use snap to determine the existence of elements")
        poco_find_snap.find_ele_by_snap(poco_instance, config, optional)
    else:
        poco_ele.wait_exists(poco_instance, selector_str, optional)


def ele_not_exit(context, param):
    poco_instance = gr.get_value("pocoInstance")
    param_dict = dsl_helper.params_to_dic(param)
    selector_str = param_dict["selector"]
    optional = {}
    if "path" in param_dict.keys():
        optional["path"] = param_dict["path"]
    elif "multiSelector" in param_dict.keys():
        optional["multiSelector"] = param_dict["multiSelector"]

    poco_ele.not_exist(poco_instance, selector_str, optional)


def wait_ele_disappear(context, param):
    poco_instance = gr.get_value("pocoInstance")
    param_dict = dsl_helper.params_to_dic(param)
    selector_str = param_dict["selector"]
    optional = {}
    if "path" in param_dict.keys():
        optional["path"] = param_dict["path"]
    elif "multiSelector" in param_dict.keys():
        optional["multiSelector"] = param_dict["multiSelector"]
    if "timeout" in param_dict.keys():
        optional["timeout"] = float(param_dict["timeout"])
    else:
        optional["timeout"] = gr.get_frame_config_value(
            "wait_ele_disappear", 10
        )

    poco_ele.wait_disappear(poco_instance, selector_str, optional)


def ele_text_equal(context, param1, param2):
    poco_instance = gr.get_value("pocoInstance")
    pattern = re.compile(r"\s+")
    param1_dict = dsl_helper.params_to_dic(param1)
    selector_str = param1_dict["selector"]
    optional = {}
    config = None
    # use snap
    # use_snap = gr.get_frame_config_value("use_snap", False)
    if "path" in param1_dict.keys():
        optional["path"] = param1_dict["path"]
    elif "multiSelector" in param1_dict.keys():
        optional["multiSelector"] = param1_dict["multiSelector"]
    if "timeout" in param1_dict.keys():
        optional["timeout"] = float(param1_dict["timeout"])
    else:
        optional["timeout"] = gr.get_frame_config_value("wait_ele_timeout", 10)

    params_deal_module = None
    deal_method = None
    param2_dict = dsl_helper.params_to_dic(param2, "tText")
    if "dealMethod" in param2_dict.keys():
        deal_method = param2_dict["dealMethod"]
        params_deal_module = gr.get_value("projectScript").params_deal
    target_str = param2_dict["tText"]
    handled_target_str = re.sub(pattern, "", target_str.replace(u"\u200b", ""))
    handled_selector_str = re.sub(
        pattern, "", selector_str.replace(u"\u200b", "")
    )
    if not (deal_method is None):
        deal_method_fun = getattr(params_deal_module, deal_method)
        handled_target_str = deal_method_fun(handled_target_str)
        pattern = re.compile(r"\s+")
        handled_target_str = re.sub(pattern, "", handled_target_str)
    if False:
        config = {"name": selector_str, "expect_text": target_str}
        poco_find_snap.verify_ele_by_snap(poco_instance, config, optional)
    else:
        poco_verify.ele_text_is(
            poco_instance,
            handled_selector_str,
            handled_target_str,
            optional,
            deal_method,
            params_deal_module,
        )


def ele_text_container(context, param1, param2):
    poco_instance = gr.get_value("pocoInstance")

    param1_dict = dsl_helper.params_to_dic(param1)
    selector_str = param1_dict["selector"]
    optional = {}
    if "path" in param1_dict.keys():
        optional["path"] = param1_dict["path"]
    elif "multiSelector" in param1_dict.keys():
        optional["multiSelector"] = param1_dict["multiSelector"]
    if "timeout" in param1_dict.keys():
        optional["timeout"] = float(param1_dict["timeout"])
    else:
        optional["timeout"] = gr.get_frame_config_value("wait_ele_timeout", 10)

    deal_method = None
    params_deal_module = None
    param2_dict = dsl_helper.params_to_dic(param2, "tText")
    if "dealMethod" in param2_dict.keys():
        deal_method = param2_dict["dealMethod"]
        params_deal_module = gr.get_value("projectScript").params_deal
    target_str = param2_dict["tText"]
    pattern = re.compile(r"\s+")
    handled_target_str = re.sub(pattern, "", target_str.replace(u"\u200b", ""))
    if not (deal_method is None):
        deal_method_fun = getattr(params_deal_module, deal_method)
        handled_target_str = deal_method_fun(handled_target_str)
        pattern = re.compile(r"\s+")
        handled_target_str = re.sub(pattern, "", handled_target_str)

    poco_verify.ele_text_contains(
        poco_instance,
        selector_str,
        handled_target_str,
        optional,
        deal_method,
        params_deal_module,
    )


def wait_ele_appear(context, param):
    poco_instance = gr.get_value("pocoInstance")

    param_dict = dsl_helper.params_to_dic(param)
    selector_str = param_dict["selector"]
    optional = {}
    if "path" in param_dict.keys():
        optional["path"] = param_dict["path"]
    elif "multiSelector" in param_dict.keys():
        optional["multiSelector"] = param_dict["multiSelector"]
    if "timeout" in param_dict.keys():
        optional["timeout"] = float(param_dict["timeout"])
    else:
        optional["timeout"] = gr.get_frame_config_value(
            "page_render_timeout", 30
        )

    poco_ele.wait_exists(poco_instance, selector_str, optional)


def exist_ele(context, param):
    """
    Compatible with the old version of the statement, it will be discarded in
    the future.
    """
    poco_instance = gr.get_value("pocoInstance")

    params_array = param.split(",")
    selector_str = params_array[0]
    optional = {}
    if len(params_array) >= 2:
        optional["timeout"] = float(params_array[1])
    else:
        optional["timeout"] = gr.get_frame_config_value(
            "wait_ele_timeout", 10
        )
    poco_ele.wait_exists(poco_instance, selector_str, optional)



# -*- coding: utf-8 -*-
"""
ios screen record
"""
from flybirds.core import global_resource as gr

__open__ = ["ScreenRecordInfo"]


class ScreenRecordInfo():
    name = "ios_screen_record"
    instantiation_timing = "plugin"

    def __init__(self):
        self.support = False
        self.start_time = None
        self.end_time = None
        self.process = None
        # 0 Just created 1 Reset state 2 Start recording
        self.status = 0

        self.use_airtest_record = gr.get_frame_config_value(
            "use_airtest_record", False
        )

    def reset(self):
        """
        Reset object
        """
        self.start_time = None
        self.end_time = None
        self.status = 1

    def destroy(self):
        """
        Destroy all recorded video processes after the task runs
        """
        pass

    def record_support(self):
        pass

    def start_record(self, timeout, bit_rate_level=1, bit_rate=None):
        """
        Start recording
        """
        pass

    def stop_record(self):
        """
        Stop the current screen recording
        """
        return None

    def link_record(self, scenario, step_index):
        """
        Associate screenshots to report
        """
        pass

    def copy_record(self, save_path):
        """
        Copy the screen recording from the mobile phone to the current client
        """
        pass

    def clear_record(self):
        """
        Delete the existing screen recording file
        """
        pass



# -*- coding: utf-8 -*-
"""
This module defines the steps related to the page.
"""

from behave import step

from flybirds.core.global_context import GlobalContext as g_Context
from flybirds.utils.dsl_helper import get_params


@step("go to url[{param}]")
def jump_to_page(context, param=None):
    (param_1,) = get_params(context, (param, "param"))
    g_Context.step.jump_to_page(context, param_1)


@step("return to previous page")
def return_pre_page(context):
    g_Context.step.return_pre_page(context)


@step("go to home page")
def to_app_home(context):
    g_Context.step.to_app_home(context)


@step("logon account[{param1}]password[{param2}]")
def app_login(context, param1=None, param2=None):
    param_1, param_2 = get_params(
        context, (param1, "param1"), (param2, "param2")
    )
    g_Context.step.app_login(context, param_1, param_2)


@step("logout")
def app_logout(context):
    g_Context.step.app_logout(context)



# -*- coding: utf-8 -*-
"""
This module is used for custom exceptions
"""


class CustomApiException(Exception):

    def __init__(self, api_name, result_code):
        message = 'platform interface{} has failed，resultCode: {}' \
            .format(api_name, result_code)
        super(CustomApiException, self).__init__(message)
        self.message = message

    def __str__(self):
        return str(self.message)



# -*- coding: utf-8 -*-
"""
Poco selector api
"""


def create_poco_object(poco, select_dic={}):
    """
    create page elements corresponding to pocoUi elements
    :param poco:
    :param select_dic:  构造 pocoObject 的限定条件
    :return:
    """
    return poco(**select_dic)


def create_parent(poco_object):
    """
    create the parent element of the pocoUi element
    :param poco_object:
    :return:
    """
    return poco_object.parent()


def create_first_child(poco_object, select_dic={}):
    """
    create the first child element of the pocoUi element
    :param poco_object:
    :param select_dic:
    :return:
    """
    if len(select_dic) >= 1:
        return poco_object.child(**select_dic)
    else:
        return poco_object.child()


def create_first_offspring(poco_object, select_dic={}):
    """
    create the first descendant element of the pocoUi element
    :param poco_object:
    :param select_dic:
    :return:
    """
    if len(select_dic) >= 1:
        return poco_object.offspring(**select_dic)
    else:
        return poco_object.offspring()


def create_first_sibling(poco_object, select_dic={}):
    """
    create the first sibling element of the pocoUi element
    :param poco_object:
    :param select_dic:
    :return:
    """
    if len(select_dic) >= 1:
        return poco_object.sibling(**select_dic)
    else:
        return poco_object.sibling()


def select_child(poco_object, target_index, select_dic={}):
    """
    select the child element of the pocoUi element
    if pocoUi does not exist on the page or the pocoUi element does not have
    any child elements or the number of child elements is less than
    target_index,an exception is thrown.
    :param poco_object:
    :param target_index:
    :param select_dic:
    :return:
    """
    return poco_object.child(**select_dic)[target_index]


def select_offspring(poco_object, target_index, select_dic={}):
    """
    select which descendant element of pocoUi element
    if pocoUi does not exist on the page or the pocoUi element does not have
    any descendant elements or the number of descendant elements is less than
    target_index, an exception is thrown.
    :param poco_object:
    :param target_index:
    :param select_dic:
    :return:
    """
    return poco_object.offspring(**select_dic)[target_index]


def select_sibling(poco_object, target_index, select_dic={}):
    """
    select which sibling element of pocoUi element
    if pocoUi does not exist on the page or the pocoUi element does not have
    any sibling elements or the number of sibling elements is less than
    target_index, an exception is thrown.
    :param poco_object:
    :param target_index:
    :param select_dic:
    :return:
    """
    return poco_object.sibling(**select_dic)[target_index]



# -*- coding: utf-8 -*-
import flybirds.core.dsl.step.step_loader
import flybirds.core.dsl.step.app
import flybirds.core.dsl.step.common
import flybirds.core.dsl.step.element
import flybirds.core.dsl.step.page
import flybirds.core.dsl.step.device

# import custom step statements
import pscript.dsl.step.custom_test

# here import extend step statements


# -*- coding: utf-8 -*-
"""
report gen factory
"""
from flybirds.utils import flybirds_log as log


class GenFactory:
    """
    gen factory
    """
    gen_factories = {}

    @staticmethod
    def gen(gen_type, report_path):
        """
        auto gen
        """
        if gen_type is not None and GenFactory.gen_factories.__contains__(
                gen_type):

            GenFactory.gen_factories[gen_type].gen(report_path)
        else:
            log.info("not have this kind of report type")

    @classmethod
    def add(cls, gen_obj):
        """
        add report gen
        """
        if hasattr(gen_obj, "name"):
            cls.gen_factories[gen_obj.name] = gen_obj



# -*- coding: utf-8 -*-
"""
This module defines the common steps.
"""

from behave import step

from flybirds.core.global_context import GlobalContext as g_Context
from flybirds.utils.dsl_helper import get_params


@step("wait[{param}]seconds")
def sleep(context, param=None):
    (param_1,) = get_params(context, (param, "param"))
    g_Context.step.sleep(context, param_1)


@step("screenshot")
def screenshot(context):
    g_Context.step.screenshot(context)


@step("information association of failed operation,"
      " run the {param1} time :[{param2}]"
      )
def prev_fail_scenario_relevance(context, param1=None, param2=None):
    param_1, param_2 = get_params(
        context, (param1, "param1"), (param2, "param2")
    )
    g_Context.step.prev_fail_scenario_relevance(context, param_1, param_2)



# -*- coding: utf-8 -*-
"""
app or other prepare
"""
import traceback
import flybirds.core.global_resource as gr
import flybirds.utils.flybirds_log as log
from flybirds.core.global_context import GlobalContext
import flybirds.core.driver.device as device
from flybirds.core.driver import app


class OnPrepare:
    """
    prepare
    """
    name = "On_App_Prepare"
    order = 21
    _app_name = "installer.apk"

    @staticmethod
    def check_config():
        """
        only local and has package path
        """
        run_info = gr.get_run_info()
        has_local = False
        has_pkg_path = False
        is_rerun = False
        if run_info is not None and run_info.run_at is not None \
                and run_info.run_at.lower() == "local":
            has_local = True

        if run_info is not None and run_info.is_rerun is not None:
            is_rerun = run_info.is_rerun

        package_path = gr.get_app_config_value("package_path")
        if package_path is not None and package_path.strip() != "":
            has_pkg_path = True
        else:
            has_pkg_path = False
        if has_local is True and has_pkg_path is True and is_rerun is False:
            return True
        else:
            return False

    @staticmethod
    def is_web_url(package_path):
        return package_path.startswith(("http:", "https:"))

    @staticmethod
    def check_app():
        """
        check whether app is installed
        """
        has_install = False
        over_install = gr.get_app_config_value("overwrite_installation")
        if over_install is True:
            return True

        pkg_name = gr.get_app_package_name()
        cmd_package_installed = "pm list packages"
        cmd_result = device.use_shell(cmd_package_installed)
        if cmd_result is not None:
            lines = cmd_result.splitlines()
            for line in lines:
                if line is not None and line:
                    ls = line.split(":")
                    if len(ls) == 2 and pkg_name == ls[1].strip():
                        log.info(f"app {pkg_name} has already be installed")
                        has_install = True
        if has_install is True:
            return False
        else:
            return True

    @staticmethod
    def can(context):
        """
        only native android can be run
        """
        if gr.get_platform() is not None and gr.get_platform().lower() \
                == "android":
            # check config
            config_check = OnPrepare.check_config()
            if config_check is True:
                return OnPrepare.check_app()
            else:
                return False
        else:
            return False

    @staticmethod
    def run(context):
        """
        install app
        """
        try:
            if hasattr(context, "installer_path"):
                installer = context.installer_path
                log.info(f"install app start, path:{installer}")
                app.install_app(installer)

                log.info("install end")
            else:
                raise Exception("not find any installer path")
        except Exception:
            log.info("fail to install app")
            exception_str = traceback.format_exc()
            if exception_str is not None and exception_str.find(
                    "INSTALL_FAILED_VERSION_DOWNGRADE") > 0:
                raise Exception(
                    "A higher version of the app is already installed on "
                    "the device, please uninstall it before installing")
            else:
                raise Exception(exception_str)


# add android prepare into global context
var = GlobalContext.join("before_run_processor", OnPrepare, 1)



# -*- coding: utf-8 -*-
"""
load event
"""
import base64

from flybirds.core.global_context import GlobalContext
from flybirds.core.plugin.plugin_manager import DirectoryPluginManager
from flybirds.core.config_manage import PluginConfig
from flybirds.core.config_manage import DeviceConfig


class PluginManager:  # pylint: disable=too-few-public-methods
    """
    plugin load management
    """

    name = "PluginManager"
    order = 1

    @staticmethod
    def run(context):
        """
        load plugin from pkg
        """
        user_data = {}
        if context is not None and context.config is not None:
            user_data = (
                context.config.userdata
                if isinstance(context.config.userdata, dict)
                else {}
            )
            for key, value in user_data.items():
                user_data[key] = str(base64.b64decode(value), "utf-8")

        p_info = PluginConfig(user_data).plugin_info
        if p_info is not None and p_info.__contains__("active"):
            GlobalContext.active_plugin = p_info.get("active")

        GlobalContext.plugin_info = p_info.get(GlobalContext.active_plugin)

        if GlobalContext.plugin_info is None:
            raise Exception(
                f"not exist this plugin {GlobalContext.active_plugin}"
            )

        GlobalContext.platform = DeviceConfig(user_data, None).platform
        plugin_manager = DirectoryPluginManager()
        plugin_manager.load_plugins()

        return plugin_manager


# add plugin load event to global processor
GlobalContext.join("plugin_processor", PluginManager, 1)



# -*- coding: utf-8 -*-
import os
import base64
import json
import flybirds.utils.file_helper as file_helper
import flybirds.utils.flybirds_log as log


def get_rerun_params(
    run_count, rerun_feature_path, max_retry_count, report_dir_path, run_args
):
    if run_args is not None:
        use_define = run_args.get("use_define")
        env_config = run_args.get("env_config")
        report_format = run_args.get("report_format")

        rerun_report_path = "{}{}rerun{}{}result{}report.json".format(
            report_dir_path,
            os.sep,
            run_count,
            os.sep,
            os.sep,
        )
        #   create and clear report path
        rerun_report_dir_path = rerun_report_path[
            0: rerun_report_path.rfind(os.sep)
        ]
        file_helper.create_dirs(rerun_report_dir_path)
        file_helper.clear_dirs(rerun_report_dir_path)
        rerun_report_path = "-o {}".format(rerun_report_path)
        log.info("reruned case report path: {}".format(rerun_report_path))

        rerun_cmd_array = [
            "behave",
            rerun_feature_path,
            report_format,
            rerun_report_path,
        ]

        # create and clear screenshot path
        rerun_screen_shot_dir = "{}{}screenshot".format(
            rerun_report_dir_path, os.sep
        )
        file_helper.create_dirs(rerun_screen_shot_dir)
        file_helper.clear_dirs(rerun_screen_shot_dir)
        rerun_user_data_screen_dir = "screenShotDir={}".format(
            str(
                base64.b64encode(rerun_screen_shot_dir.encode("utf-8")),
                "utf-8",
            )
        )
        screen_data_index = -1
        for use_data_index in range(len(use_define)):
            if use_define[use_data_index].startswith("screenShotDir="):
                screen_data_index = use_data_index
                break
        if screen_data_index != -1:
            use_define[screen_data_index] = rerun_user_data_screen_dir
        else:
            use_define.extend(
                [
                    "--define",
                    rerun_user_data_screen_dir,
                ]
            )
        log.info(
            "reruned case screenshot path: {}".format(rerun_screen_shot_dir)
        )

        is_rerun = "flybirdsAutoRerun={}".format(
            str(
                base64.b64encode("Yes".encode("utf-8")),
                "utf-8",
            )
        )
        run_path_array = []
        if run_count > 1:
            for item_count in range(1, run_count + 1):
                run_path_array.append(
                    str(
                        os.path.join(
                            report_dir_path,
                            "rerun" + str(item_count),
                            "fail_relevance.json",
                        )
                    )
                )
            re_run_info = "flybirdsAutoRerunInfo={}".format(
                str(
                    base64.b64encode(",".join(run_path_array).encode("utf-8")),
                    "utf-8",
                )
            )
        else:
            re_run_info = "flybirdsAutoRerunInfo={}".format(
                str(
                    base64.b64encode(
                        str(
                            os.path.join(
                                report_dir_path,
                                "rerun" + str(run_count),
                                "fail_relevance.json",
                            )
                        ).encode("utf-8")
                    ),
                    "utf-8",
                )
            )
        use_define.extend(["--define", is_rerun])
        use_define.extend(["--define", re_run_info])
        rerun_cmd_array.extend(
            [
                "--define maxRetryCount={}".format(
                    str(
                        base64.b64encode(str(max_retry_count).encode("utf-8")),
                        "utf-8",
                    )
                )
            ]
        )
        if len(use_define) > 0:
            rerun_cmd_array.extend(use_define)
        if not (env_config is None):
            rerun_cmd_array.extend(
                [
                    "--define es={}".format(
                        str(
                            base64.b64encode(
                                json.dumps(env_config).encode("utf-8")
                            ).decode("utf-8")
                        )
                    )
                ]
            )

        rerun_cmd_array.append(
            "--no-color --no-capture --no-capture-stderr --no-skipped"
        )
        rerun_cmd_str = " ".join(rerun_cmd_array)
        log.info("rerun behave execute: {}".format(rerun_cmd_str))
        return {
            "rerun_cmd_str": rerun_cmd_str,
            "rerun_report_dir_path": rerun_report_dir_path,
        }
    else:
        return None



# -*- coding: utf-8 -*-
"""
The expansion of each hook function during the execution of flybirds
"""


def before_all_extend(context):
    pass


def after_all_extend(context):
    pass


def before_feature_extend(context, feature):
    pass


def after_feature_extend(context, feature):
    pass


def before_scenario_extend(context, scenario):
    pass


def after_scenario_extend(context, scenario):
    pass


def before_step_extend(context, step):
    pass


def after_step_extend(context, step):
    pass


def before_tag_extend(context, tag):
    pass


def after_tag_extend(context, tag):
    pass



# -*- coding: utf-8 -*-
"""
app base step class
"""
import flybirds.core.plugin.plugins.default.step.app as step_app
import flybirds.core.plugin.plugins.default.step.attr as step_attr
import flybirds.core.plugin.plugins.default.step.click as step_click
import flybirds.core.plugin.plugins.default.step.common as step_common
import flybirds.core.plugin.plugins.default.step.page_show_adjust as step_adjust
import flybirds.core.plugin.plugins.default.step.record as step_record
import flybirds.core.plugin.plugins.default.step.swipe as step_swipe
import flybirds.core.plugin.plugins.default.step.verify as step_verify
from flybirds.core.plugin.plugins.default.step.input import ele_input
from flybirds.core.plugin.plugins.default.step.position import (
    position_not_change
)


class AppBaseStep:
    """APP Base Step Class"""

    name = "app_base_step"

    def init_device(self, context, param=None):
        step_app.init_device(context, param)

    def connect_device(self, context, param):
        step_app.connect_device(context, param)

    def start_app(self, context, param):
        step_app.start_app(context, param)

    def restart_app(self, context):
        step_app.restart_app(context)

    def stop_app(self, context):
        step_app.stop_app(context)

    def text_attr_equal(self, context, param1, param2, param3):
        step_attr.text_attr_equal(context, param1, param2, param3)

    def ele_attr_equal(self, context, param1, param2, param3):
        step_attr.ele_attr_equal(context, param1, param2, param3)

    def click_ele(self, context, param):
        step_click.click_ele(context, param)

    def click_text(self, context, param):
        step_click.click_text(context, param)

    def click_coordinates(self, context, x, y):
        step_click.click_coordinates(context, x, y)

    def sleep(self, context, param):
        step_common.sleep(context, param)

    def screenshot(self, context):
        step_common.screenshot(context)

    def prev_fail_scenario_relevance(self, context, param1, param2):
        step_common.prev_fail_scenario_relevance(context, param1, param2)

    def ele_input(self, context, param1, param2):
        ele_input(context, param1, param2)

    def swipe_to_ele(self, context, param1, param2, param3):
        step_adjust.swipe_to_ele(context, param1, param2, param3)

    def full_screen_swipe_to_ele_aaa(self, context, param1, param2):
        step_adjust.full_screen_swipe_to_ele_aaa(context, param1, param2)

    def position_not_change(self, context, param1, param2):
        position_not_change(context, param1, param2)

    def start_screen_record_timeout(self, context, param):
        step_record.start_screen_record_timeout(context, param)

    def start_screen_record(self, context):
        step_record.start_screen_record(context)

    def stop_screen_record(self, context):
        step_record.stop_screen_record(context)

    def ele_swipe(self, context, param1, param2, param3):
        step_swipe.ele_swipe(context, param1, param2, param3)

    def full_screen_swipe(self, context, param1, param2):
        step_swipe.full_screen_swipe(context, param1, param2)

    def wait_text_exist(self, context, param):
        step_verify.wait_text_exist(context, param)

    def text_not_exist(self, context, param):
        step_verify.text_not_exist(context, param)

    def wait_text_disappear(self, context, param):
        step_verify.wait_text_disappear(context, param)

    def wait_ele_exit(self, context, param):
        step_verify.wait_ele_exit(context, param)

    def ele_not_exit(self, context, param):
        step_verify.ele_not_exit(context, param)

    def wait_ele_disappear(self, context, param):
        step_verify.wait_ele_disappear(context, param)

    def ele_text_equal(self, context, param1, param2):
        step_verify.ele_text_equal(context, param1, param2)

    def ele_text_container(self, context, param1, param2):
        step_verify.ele_text_container(context, param1, param2)

    def wait_ele_appear(self, context, param):
        step_verify.wait_ele_appear(context, param)

    def exist_ele(self, context, param):
        step_verify.exist_ele(context, param)



# -*- coding: utf-8 -*-

"""
config management all config should load by this module
"""
import os

from flybirds.utils import file_helper
from flybirds.utils import flybirds_log as log
from flybirds.utils.dsl_helper import return_value


class ConfigManage:
    """
    Manage all configuration information
    """

    def __init__(self, user_data):
        self.run_config = RunConfig(user_data)
        log.info(
            "run_config", str(self.run_config.__dict__)
        )

        self.log_config = LogConfig(user_data, None)
        log.info(
            "Logging configuration information", str(self.log_config.__dict__)
        )
        self.app_info = AppConfig(user_data, None)
        log.info("APP configuration information", str(self.app_info.__dict__))
        self.device_info = DeviceConfig(user_data, None)
        log.info(
            "Device configuration information", str(self.device_info.__dict__)
        )
        self.frame_info = FrameConfig(user_data, None)
        log.info(
            "Frame parameter configuration information",
            str(self.frame_info.__dict__),
        )
        self.schema_info = SchemaUrl()
        self.report_info = ReportConfig(user_data, None)
        log.info(
            "Test report configuration information",
            str(self.report_info.__dict__),
        )
        self.flow_behave = FlowBehave(user_data, None)
        log.info(
            "Process control configuration information",
            str(self.flow_behave.__dict__),
        )


def get_config(config, name):
    """
    load flybirds config
    """
    if config is None:
        path = os.path.join(os.getcwd(), "config", "flybirds_config.json")
        if os.path.exists(path):
            c_f = file_helper.get_json_from_file(path)
            if c_f.__contains__(name):
                return c_f.get(name)
    elif config.__contains__(name):
        return config.get(name)

    return None


class AppConfig:
    """
    Read configuration information about the test app

    Attributes:
        package_name: the packageName of the app on the android system
        unique_tag: the unique identifier of the app in the business system
    """

    def __init__(self, user_data, config=None):
        app_config = get_config(config, "app_info")
        if app_config is None:
            path = os.path.join(os.getcwd(), "config", "app_info.json")
            if os.path.exists(path):
                app_config = file_helper.get_json_from_file(path)

        if app_config is not None:
            self.package_name = user_data.get(
                "packageName", app_config["packageName"]
            )
            self.unique_tag = user_data.get(
                "uniqueTag", app_config.get("uniqueTag", None)
            )
            self.default_user = user_data.get(
                "defaultUser", app_config.get("defaultUser", None)
            )
            self.default_password = user_data.get(
                "defaultPassword", app_config.get("defaultPassword", None)
            )
            self.user_group = user_data.get(
                "userGroup", app_config.get("userGroup", 1)
            )

            self.package_path = user_data.get(
                "packagePath", app_config.get("packagePath", None)
            )

            self.overwrite_installation = user_data.get(
                "overwriteInstallation",
                app_config.get("overwriteInstallation", None)
            )

            self.run_id = user_data.get("runId", None)
            self.build_id = user_data.get("buildId", None)
            self.pkg_version = user_data.get("pkgVersion", None)


class AppEnvConfig:
    """
    Manage all configuration information
    """

    def __init__(self, user_data, config=None):
        self.log_config = LogConfig(user_data, config)
        log.info(
            "Logging configuration information", str(self.log_config.__dict__)
        )
        self.env_config = user_data.get("es", None)


class DeviceConfig:
    """
    Read configuration information about the test device (mobile phone)

    Attributes:
        device_id: the unique identifier of the phone
    """

    def __init__(self, user_data, config):
        device_info = get_config(config, "device_info")
        if device_info is None:
            path = os.path.join(os.getcwd(), "config", "device_config.json")
            if os.path.exists(path):
                device_info = file_helper.get_json_from_file(path)

        if device_info is not None:
            self.device_id = user_data.get("deviceId", device_info["deviceId"])
            platform = "android"
            device_driver = None
            if device_info.__contains__("platform"):
                platform = device_info["platform"]
            if device_info.__contains__("webDriverAgent"):
                device_driver = device_info["webDriverAgent"]

            self.platform = user_data.get("platform", platform)
            self.web_driver_agent = device_driver
            self.screen_size = None


class FlowBehave:
    """
    Read some function switches
    """

    def __init__(self, user_data, config):
        flow_config = get_config(config, "flow_behave")
        if flow_config is None:
            path = os.path.join(os.getcwd(), "config", "flow_behave.json")
            if os.path.exists(path):
                flow_config = file_helper.get_json_from_file(path)

        if flow_config is not None:
            self.before_run_page = user_data.get(
                "beforeRunPage",
                return_value(flow_config.get("beforeRunPage", "restartApp"),
                             "restartApp")
            )
            self.scenario_fail_page = user_data.get(
                "scenarioFailPage",
                return_value(flow_config.get("scenarioFailPage", "restartApp"),
                             "restartApp")
            )
            self.scenario_success_page = user_data.get(
                "scenarioSuccessPage",
                return_value(
                    flow_config.get("scenarioSuccessPage", "backupPage"),
                    "backupPage")
            )
            self.before_run_login = user_data.get(
                "beforeRunLogin",
                return_value(flow_config.get("beforeRunLogin", False), False)
            )
            self.fail_screen_record = user_data.get(
                "failScreenRecord",
                return_value(flow_config.get("failScreenRecord", False), False)
            )
            self.scenario_screen_record_time = user_data.get(
                "scenarioScreenRecordTime",
                return_value(flow_config.get("scenarioScreenRecordTime", 120),
                             120)
            )

            self.fail_rerun = return_value(flow_config.get("failRerun", False),
                                           False)
            self.max_fail_rerun_count = return_value(flow_config.get(
                "maxFailRerunCount", 1.0), 1.0)

            # Maximum number of retries
            self.max_retry_count = user_data.get(
                "maxRetryCount",
                return_value(flow_config.get("maxRetryCount", 1), 1)
            )
        if not hasattr(self, "before_run_page"):
            self.before_run_page = user_data.get("beforeRunPage", "restartApp")
        if not hasattr(self, "scenario_fail_page"):
            self.scenario_fail_page = user_data.get(
                "scenarioFailPage", "restartApp"
            )
        if not hasattr(self, "scenario_success_page"):
            self.scenario_success_page = user_data.get(
                "scenarioSuccessPage", "backupPage"
            )
        if not hasattr(self, "before_run_login"):
            self.before_run_login = user_data.get("beforeRunLogin", False)
        if not hasattr(self, "fail_screen_record"):
            self.fail_screen_record = user_data.get("failScreenRecord", False)
        if not hasattr(self, "scenario_screen_record_time"):
            self.scenario_screen_record_time = user_data.get(
                "scenarioScreenRecordTime", 120
            )

        if not hasattr(self, "fail_rerun"):
            self.fail_rerun = False
        if not hasattr(self, "max_fail_rerun_count"):
            self.max_fail_rerun_count = 1.0
        if not hasattr(self, "max_retry_count"):
            self.max_retry_count = 1


class FrameConfig:
    """
    Read some configurations used by the UI framework
    """

    def __init__(self, user_data, config):
        frame_config = get_config(config, "frame_info")
        if frame_config is None:
            path = os.path.join(os.getcwd(), "config", "frame_info.json")
            if os.path.exists(path):
                frame_config = file_helper.get_json_from_file(path)
        if frame_config is not None:
            self.wait_ele_timeout = user_data.get(
                "waitEleTimeout",
                return_value(frame_config.get("waitEleTimeout", 10), 10)
            )
            self.wait_ele_disappear = user_data.get(
                "waitEleDisappear",
                return_value(frame_config.get("waitEleDisappear", 10), 10)
            )
            self.click_verify_timeout = user_data.get(
                "clickVerifyTimeout",
                return_value(frame_config.get("clickVerifyTimeout", 10), 10)
            )
            self.use_swipe_duration = user_data.get(
                "useSwipeDuration",
                return_value(frame_config.get("useSwipeDuration", False),
                             False)
            )
            self.swipe_duration = user_data.get(
                "swipeDuration",
                return_value(frame_config.get("swipeDuration", 1), 1)
            )
            self.use_poco_input = user_data.get(
                "usePocoInput",
                return_value(frame_config.get("usePocoInput", False), False)
            )
            self.after_input_wait = user_data.get(
                "afterInputWait",
                return_value(frame_config.get("afterInputWait", 1), 1)
            )
            self.use_search_swipe_duration = user_data.get(
                "useSearchSwipeDuration",
                return_value(frame_config.get("useSearchSwipeDuration", False),
                             False)
            )
            self.search_swipe_duration = user_data.get(
                "searchSwipeDuration",
                return_value(frame_config.get("searchSwipeDuration", 1), 1)
            )
            self.swipe_search_count = user_data.get(
                "swipeSearchCount",
                return_value(frame_config.get("swipeSearchCount", 5), 5)
            )
            self.swipe_search_distance = user_data.get(
                "swipeSearchDistance",
                return_value(frame_config.get("swipeSearchDistance", 0.3), 0.3)
            )
            self.page_render_timeout = user_data.get(
                "pageRenderTimeout",
                return_value(frame_config.get("pageRenderTimeout", 30), 30)
            )
            self.app_start_time = user_data.get(
                "appStartTime",
                return_value(frame_config.get("appStartTime", 6), 6)
            )
            self.swipe_ready_time = user_data.get(
                "swipeReadyTime",
                return_value(frame_config.get("swipeReadyTime", 3), 3)
            )
            self.verify_pos_not_change_count = user_data.get(
                "verifyPosNotChangeCount",
                return_value(frame_config.get("verifyPosNotChangeCount", 5), 5)
            )
            self.screen_record_time = user_data.get(
                "screenRecordTime",
                return_value(frame_config.get("screenRecordTime", 60), 60)
            )
            # Snapshot method
            self.use_snap = user_data.get(
                "useSnap",
                return_value(frame_config.get("useSnap", False), False)
            )
            # Whether to use airtest record
            self.use_airtest_record = user_data.get(
                "useAirtestRecord",
                return_value(frame_config.get("useAirtestRecord", False),
                             False)
            )
        self.set_frame_info_attrs(user_data)
        self.set_other_attrs(user_data)

    def set_frame_info_attrs(self, user_data):
        if not hasattr(self, "wait_ele_timeout"):
            self.wait_ele_timeout = user_data.get("waitEleTimeout", 10)
        if not hasattr(self, "wait_ele_disappear"):
            self.wait_ele_disappear = user_data.get("waitEleDisappear", 10)
        if not hasattr(self, "click_verify_timeout"):
            self.click_verify_timeout = user_data.get("clickVerifyTimeout", 10)
        if not hasattr(self, "use_swipe_duration"):
            self.use_swipe_duration = user_data.get("useSwipeDuration", False)
        if not hasattr(self, "swipe_duration"):
            self.swipe_duration = user_data.get("swipeDuration", 1)
        if not hasattr(self, "use_poco_input"):
            self.use_poco_input = user_data.get("usePocoInput", False)
        if not hasattr(self, "after_input_wait"):
            self.after_input_wait = user_data.get("afterInputWait", 1)
        if not hasattr(self, "use_search_swipe_duration"):
            self.use_search_swipe_duration = user_data.get(
                "useSearchSwipeDuration", False
            )

    def set_other_attrs(self, user_data):
        if not hasattr(self, "search_swipe_duration"):
            self.search_swipe_duration = user_data.get(
                "searchSwipeDuration", 1
            )
        if not hasattr(self, "swipe_search_count"):
            self.swipe_search_count = user_data.get("swipeSearchCount", 5)
        if not hasattr(self, "swipe_search_distance"):
            self.swipe_search_distance = user_data.get(
                "swipeSearchDistance", 0.3
            )
        if not hasattr(self, "page_render_timeout"):
            self.page_render_timeout = user_data.get("pageRenderTimeout", 30)
        if not hasattr(self, "app_start_time"):
            self.app_start_time = user_data.get("appStartTime", 6)
        if not hasattr(self, "swipe_ready_time"):
            self.swipe_ready_time = None
        if not hasattr(self, "verify_pos_not_change_count"):
            self.verify_pos_not_change_count = user_data.get(
                "verifyPosNotChangeCount", 6
            )
        if not hasattr(self, "screen_record_time"):
            self.screen_record_time = user_data.get("screenRecordTime", 60)
        if not hasattr(self, "use_snap"):
            self.use_snap = user_data.get("useSnap", False)
        if not hasattr(self, "use_airtest_record"):
            self.use_airtest_record = user_data.get("useAirtestRecord", False)


class LogConfig:
    """
    Read the related configuration of the log
    """

    def __init__(self, user_data, config):
        log_config = get_config(config, "log")
        if log_config is None:
            path = os.path.join(os.getcwd(), "config", "log_config.json")
            if os.path.exists(path):
                log_config = file_helper.get_json_from_file(path)

        if log_config is not None:
            self.level = user_data.get(
                "logLevel",
                return_value(log_config.get("logLevel", "info"), "info")
            )

        if (not hasattr(self, "level")) or (not isinstance(self.level, str)):
            self.level = user_data.get("logLevel", "info")


class ReportConfig:
    """
    Some configurations of the report
    """

    def __init__(self, user_data, config):
        report_config = get_config(config, "report")
        if report_config is None:
            path = os.path.join(os.getcwd(), "config", "report_config.json")
            if os.path.exists(path):
                report_config = file_helper.get_json_from_file(path)
        if report_config is not None:
            self.screen_shot_dir = user_data.get(
                "screenShotDir", report_config.get("screenShotDir", None)
            )

        if not hasattr(self, "screen_shot_dir"):
            self.screen_shot_dir = user_data.get("screenShotDir", None)


class SchemaUrl:
    """
    All schema urls used
    """

    def __init__(self):
        schema_url_path = os.path.join(
            os.getcwd(), "config", "schema_url.json"
        )
        if os.path.exists(schema_url_path):
            self.all_schema_url = file_helper.get_json_from_file(
                schema_url_path
            )


class PluginConfig:
    """
    plugin config
    """

    def __init__(self, user_data):
        path = os.path.join(os.getcwd(), "config", "plugin_info.json")
        if os.path.exists(path):
            plugin_info = file_helper.get_json_from_file(path)
            self.plugin_info = plugin_info
        else:
            self.plugin_info = {"active": "default", "default": {}}


class RunConfig:
    """
    run base info
    """

    def __init__(self, user_data):
        is_rerun = user_data.get("flybirdsAutoRerun", None)
        if is_rerun is not None and is_rerun.lower() == "yes":
            self.is_rerun = True
        else:
            self.is_rerun = False

        run_at = user_data.get("run_at", "local")
        self.run_at = run_at



# -*- coding: utf-8 -*-
"""
android step implements class
"""

from flybirds.core.plugin.plugins.default.app_base_step import AppBaseStep
from flybirds.core.plugin.plugins.default.step.app \
    import (install_app, uninstall_app, to_app_home, app_login, app_logout,
            return_pre_page)
from flybirds.core.plugin.plugins.default.step.schema import jump_to_page

__open__ = ["Step"]


class Step(AppBaseStep):
    """Android Step Class"""

    name = "android_step"

    def install_app(self, context, param):
        install_app(context, param)

    def uninstall_app(self, context, param):
        uninstall_app(context, param)

    def to_app_home(self, context):
        to_app_home(context)

    def app_login(self, context, param1, param2):
        app_login(context, param1, param2)

    def app_logout(self, context):
        app_logout(context)

    def return_pre_page(self, context):
        return_pre_page(context)

    def jump_to_page(self, context, param):
        jump_to_page(context, param)



# -*- coding: utf-8 -*-
"""
Device screenshot method.
"""
import os
import time
import traceback
from base64 import b64decode

import flybirds.core.global_resource as gr
import flybirds.utils.file_helper as file_helper
import flybirds.utils.flybirds_log as log
import flybirds.utils.uuid_helper as uuid_helper
from flybirds.core.global_context import GlobalContext as g_context
from flybirds.core.plugin.plugins.default.ios_snapshot import get_screen


def screen_shot(path):
    """
    Take a screenshot and save
    """
    log.info(f"screen shot start------path :{path}")
    # poco = g_context.element.ui_driver_init()
    poco = g_context.ui_driver_instance
    screen_size = gr.get_device_size()
    cur_platform = g_context.platform
    try:
        if cur_platform is not None and cur_platform.strip().lower() == "ios":
            b64img, fmt = get_screen()
        else:
            b64img, fmt = poco.snapshot(width=screen_size[1])

        open(path, "wb").write(b64decode(b64img))
    except Exception as e:
        log.warn(
            "Screenshot failed path: {}, error: {}".format(path, str(e)),
            traceback.format_exc(),
        )

    log.info("screen shot end------")


def screen_link_to_behave(scenario, step_index, tag=None):
    """
    screenshot address and linked to the <scr> tag
    The label information is placed in the description of the scene,
    and the json report is processed after all the runs are finished,
    and the <scr> information in the description is converted into embeddings
    information in the step.
    """
    feature_name = file_helper.valid_file_name(scenario.feature.name)
    scenario_name = file_helper.valid_file_name(scenario.name)

    if len(scenario.steps) > step_index >= 0:
        file_name = None
        if not (tag is None):
            file_name = tag
        file_name += (
                scenario_name
                + uuid_helper.create_short_uuid()
                + str(int(round(time.time() * 1000)))
                + ".png"
        )

        screen_shot_dir = gr.get_screen_save_dir()
        if not (screen_shot_dir is None):
            current_screen_dir = os.path.join(screen_shot_dir, feature_name)
        else:
            current_screen_dir = os.path.join(feature_name)
        file_helper.create_dirs_path_object(current_screen_dir)

        src_path = "../screenshot/{}/{}".format(feature_name, file_name)
        log.info("screen_link_to_behave: {}".format(src_path))
        data = (
            'embeddingsTags, stepIndex={}, <image class ="screenshot"'
            ' width="375" src="{}" />'.format(step_index, src_path)
        )
        scenario.description.append(data)

        log.info(f"screen_shot_dir path :{screen_shot_dir}")
        screen_shot(os.path.join(current_screen_dir, file_name))



# -*- coding: utf-8 -*-

import os
import subprocess
import traceback
import operator

from flybirds.utils import flybirds_log as log
from flybirds.report.fail_feature_create import rerun_launch
from flybirds.utils.pkg_helper import load_pkg_by_ns


class RunManage:
    """
    run cycle manage class
    """

    def __init__(self):
        pass

    before_run_processor = []
    after_run_processor = []

    @staticmethod
    def load_pkg():
        """
        load report ref
        """
        load_pkg_by_ns("flybirds.core.launch_cycle")
        load_pkg_by_ns("flybirds.report.gen")

    @staticmethod
    def process(processors_name, *args):
        """
        processor executor
        """
        processors = getattr(RunManage, processors_name)
        if processors is not None and len(processors) > 0:
            sort_key = operator.attrgetter("order")
            processors.sort(key=sort_key)
            for processor in processors:
                if hasattr(processor, "can"):
                    can_run = processor.can(*args)
                    if can_run is True:
                        processor.run(*args)
                else:
                    processor.run(*args)

    @staticmethod
    def run(context):
        """
        run logical contains : before ,exe ,after
        """
        RunManage.load_pkg()
        RunManage.process("before_run_processor", context)
        RunManage.exe(context)
        RunManage.process("after_run_processor", context)

    @staticmethod
    def exe(context):
        """
        start behave process and rerun fail case
        """
        no_args = context.get("no_args")
        if no_args is not None and no_args is False:

            # noinspection PyBroadException
            try:
                run_args = context.get("run_args")
                cmd_str = context.get("cmd_str")
                need_rerun_args = context.get("need_rerun_args")
                report_dir_path = context.get("report_dir_path")

                behave_process = subprocess.Popen(
                    cmd_str, cwd=os.getcwd(), shell=True, stdout=None
                )

                behave_process.wait()
                behave_process.communicate()

                # Determine whether the failed scenario needs to be re-run
                rerun_launch(need_rerun_args, report_dir_path, run_args)

            except Exception:
                log.error(
                    f"behave task execute error: {traceback.format_exc()}")
        else:
            log.info("cannot run non-args")

    @staticmethod
    def join(processors_name, processor, replace=0):
        """
        add processor
        """
        if processor is not None and processor.name is not None:
            processors = getattr(RunManage, processors_name)
            indx = -1
            for i, item in enumerate(processors):
                if item.name is not None and item.name == processor.name:
                    indx = i
                    break
            if replace == 0:
                if indx >= 0:
                    return indx
            elif indx >= 0:
                processors[indx] = processor
                return indx
            processors.append(processor)

            return indx
        return -1

    @staticmethod
    def insert(processors_name, processor, replace=0):
        """
        insert process
        """
        if processor is not None and processor.name is not None:
            processors = getattr(RunManage, processors_name)
            indx = -1
            for i, item in enumerate(processors):
                if item.name is not None and item.name == processor.name:
                    indx = i
                    break
            if replace == 0:
                if indx >= 0:
                    return indx
            elif indx >= 0:
                del processors[indx]
            processors.insert(0, processor)

            return indx
        return -1


def run_script(run_args):
    log.info(f"received run_args: {run_args}")
    try:
        r_context = {
            "run_args": run_args
        }
        RunManage.run(r_context)
    except Exception:
        log.error(f"behave task run error: {traceback.format_exc()}")



# -*- coding: utf-8 -*-
"""
fail scenario create rerun
"""
import json
import os
import re
import shutil
import subprocess

from flybirds.core.config_manage import FlowBehave
from flybirds.report import json_format_deal
from flybirds.report.rerun_params import get_rerun_params
from flybirds.utils import file_helper
from flybirds.utils import flybirds_log as log
from flybirds.utils import language_helper as lge
from flybirds.utils import uuid_helper


class FailScenarioSum:
    """
    Information on all failed scenarios
    """

    def __init__(self):
        self.fail_scenarios = {}

    def add_scenario(self, feature_name, scenario_name, description):
        """
        Add the failure scenario to the current failure set, the key is the
        only one
        """
        scenario_key = str(uuid_helper.create_uuid())
        while scenario_key in list(self.fail_scenarios):
            log.info(
                "When adding a failure scenario, the generated uuid is "
                "duplicated！"
            )
            scenario_key = str(uuid_helper.create_uuid())
        scenario_value = FailScenarioInfo(
            feature_name, scenario_name, description
        )
        self.fail_scenarios[scenario_key] = json.dumps(scenario_value.__dict__)
        return scenario_key

    def serialize_to_file(self, rerun_root_dir):
        """
        Serialize all the information associated with the failure scenario
        and save it in a file
        """
        file_helper.store_json_to_file_path(
            self.fail_scenarios,
            os.path.join(rerun_root_dir, "fail_relevance.json"),
            "w+",
        )


class FailScenarioInfo:
    """
    Some information about the failure scenario, which can be associated with
    the failure information in the report when re-run
    """

    def __init__(self, feature_name, scenario_name, description):
        self.feature_name = feature_name
        self.scenario_name = scenario_name
        if not isinstance(description, list):
            self.description = []
        else:
            self.description = description


def rerun_launch(need_rerun_args, report_dir_path, run_args):
    """
    start to rerun
    """
    # Determine whether the failed scenario needs to be re-run
    flow_behave_config = FlowBehave({}, None)
    rerun_report_dir_path = None
    # rerun_feature_path_array = []
    max_fail_rerun_count = flow_behave_config.max_fail_rerun_count

    if not need_rerun_args:
        need_rerun = flow_behave_config.fail_rerun
    else:
        need_rerun = need_rerun_args
    if need_rerun:
        max_retry_count = flow_behave_config.max_retry_count
        run_count = 1
        rerun_dir_path = report_dir_path
        count_satisfy = create_rerun(
            report_dir_path,
            rerun_dir_path,
            run_count,
            max_fail_rerun_count,
        )
        # The number of failures is not higher than the limited value and
        # needs to be re-run
        if count_satisfy:
            while max_retry_count > 0:
                log.info(
                    (
                        f"Number of failed retries{run_count}, start to "
                        f"generate feature files that failed to rerun, "
                        f"report root directory{report_dir_path}"
                    )
                )
                rerun_dir_path = report_dir_path
                try:
                    if run_count > 1:
                        count_satisfy = create_rerun(
                            rerun_report_dir_path,
                            rerun_dir_path,
                            run_count,
                            max_fail_rerun_count,
                        )
                    rerun_feature_path = (
                        f"{report_dir_path}{os.sep}rerun" f"{run_count}"
                    )
                    # rerun_feature_path_array.append(rerun_feature_path)
                    log.info(
                        f"count_satisfy: {count_satisfy}, max_retry_count: "
                        f"{max_retry_count}, run_count: {run_count}"
                    )
                    if count_satisfy:
                        rerun_params = get_rerun_params(
                            run_count,
                            rerun_feature_path,
                            max_retry_count,
                            report_dir_path,
                            run_args,
                        )
                        rerun_cmd_str = rerun_params.get("rerun_cmd_str")
                        rerun_report_dir_path = rerun_params.get(
                            "rerun_report_dir_path"
                        )
                        if rerun_cmd_str is not None:
                            rerun_behave_process = subprocess.Popen(
                                rerun_cmd_str,
                                cwd=os.getcwd(),
                                shell=True,
                                stdout=None,
                            )
                            rerun_behave_process.wait()
                            rerun_behave_process.communicate()
                            # Re-run of the failed scenario ends
                            # Number of reruns -1, actual number of runs +1
                            max_retry_count = max_retry_count - 1
                            run_count = run_count + 1
                            # Failed to rerun result move
                            log.info(
                                f"move_rerun_screen, report_dir_path: "
                                f"{report_dir_path}, rerun_report_dir_path:"
                                f" {rerun_report_dir_path}"
                            )
                            json_format_deal.copy_rerun_screen(
                                report_dir_path, rerun_report_dir_path
                            )
                        else:
                            log.info(
                                "Failure to re-run parameters not obtained"
                            )
                            break
                    else:
                        log.info(
                            f"Number of failed retry runs: {run_count}, "
                            f"do you need to rerun the task:{count_satisfy}, "
                            "failed retry task is not executed"
                        )
                        break
                except Exception as e:
                    log.error(f"behave failed retry error: {e}")
                    break
        else:
            log.info(
                f"Do you need to rerun the task: {count_satisfy}, the failed "
                f"retry task was not executed"
            )
        log.info(
            f"Start processing the json report under the directory "
            f"{report_dir_path}"
        )
    # if rerun_report_dir_path is not None:
    json_format_deal.parse_json_data(report_dir_path, rerun_report_dir_path)


def create_rerun(report_dir, rerun_dir, run_count, max_fail_count=1.0):
    """
    Feature to re-run after creation failure
    Some information after failure (such as screenshots) falls into the file
    , and this information is
    associated in the next run
    """
    result = True
    sum_count = 0
    fail_count = 0

    rerun_root_dir = f"{rerun_dir}{os.sep}rerun{run_count}"
    if not file_helper.create_dirs(rerun_root_dir):
        file_helper.clear_dirs(rerun_root_dir)

    fail_scenario_static = FailScenarioSum()

    rerun_feature_index = run_count
    exist_scenario_name = []

    # Second level file index
    # rerun_feature_second_index = 1

    sum_count, fail_count, fail_scenario_static, rerun_root_dir = \
        process_loop_block(report_dir, rerun_feature_index, sum_count,
                           fail_count,
                           exist_scenario_name, fail_scenario_static,
                           run_count,
                           rerun_root_dir)
    if sum_count <= 0 or fail_count <= 0:
        log.info(
            "Feature sum_count rerun after creation"
            " failure <= 0 or fail_count <= 0",
            sum_count <= 0 or fail_count <= 0,
        )
        result = False
    elif (
        isinstance(max_fail_count, float)
        and (fail_count / sum_count) > max_fail_count
    ):
        log.info(
            "Feature to rerun after creation failure "
            "(fail_count / sum_count)> max_fail_count",
            (fail_count / sum_count) > max_fail_count,
        )
        result = False
    elif isinstance(max_fail_count, int) and fail_count > max_fail_count:
        log.info(
            "Feature fail_count> max_fail_count to"
            " rerun after creation failure",
            fail_count > max_fail_count,
        )
        result = False

    fail_scenario_static.serialize_to_file(rerun_root_dir)
    copy_behave_need_file(rerun_root_dir)
    return result


def process_loop_block(report_dir, rerun_feature_index, sum_count, fail_count,
                       exist_scenario_name, fail_scenario_static, run_count,
                       rerun_root_dir):
    for file_item in os.listdir(report_dir):
        if re.search(r"\.json", str(file_item)) is not None:
            # noinspection PyBroadException
            try:
                file_path = os.path.join(report_dir, file_item)
                report_json = file_helper.get_json_from_file_path(file_path)
                if isinstance(report_json, list):
                    for feature in report_json:
                        cur_feature_array = get_init_feature_array(
                            rerun_feature_index, feature.get("language")
                        )
                        rerun_feature_location = feature["location"]
                        rerun_match_obj = re.match(
                            r"(.*\/)*([^.]+).feature", rerun_feature_location
                        )

                        # Because the path of the failed file should be
                        # displayed in the report,
                        # the failure case is named using the original feature,
                        # not integrated into one file

                        rerun_feature_name = (
                            f"flybirdsARFeature" f"{rerun_feature_index}"
                        )
                        if rerun_match_obj is not None:
                            rerun_feature_name = rerun_match_obj.group(2)
                        log.info(f"rerun_feature_name: {rerun_feature_name}")
                        if isinstance(feature.get("elements", None), list):
                            for scenario in feature["elements"]:
                                if scenario["type"] == "background":
                                    continue
                                sum_count += 1
                                if scenario["status"] == "failed":
                                    fail_count += 1
                                    if isinstance(scenario["tags"], list):
                                        cur_tags = ""
                                        for tag_item in scenario["tags"]:
                                            cur_tags += f" @{tag_item}"
                                        cur_feature_array.append(
                                            f" {cur_tags}\n"
                                        )
                                    rerun_scenario_name = scenario["name"]
                                    while (
                                            rerun_scenario_name
                                            in exist_scenario_name
                                    ):
                                        rerun_scenario_name += str(range(9))
                                    exist_scenario_name.append(
                                        rerun_scenario_name
                                    )
                                    log.info(
                                        "exist_scenario_name: "
                                        f"{str(exist_scenario_name)}"
                                    )
                                    f_language = feature.get("language")
                                    scenario_key = lge.parse_keyword(
                                        "scenario", f_language
                                    )

                                    cur_feature_array.append(
                                        f"  {scenario_key}: "
                                        f"{rerun_scenario_name}\n"
                                    )

                                    relevance_key = (
                                        fail_scenario_static.add_scenario(
                                            feature["name"],
                                            scenario["name"],
                                            scenario["description"],
                                        )
                                    )
                                    if relevance_key:
                                        g_step_name = lge.parse_glb_str(
                                            "information association of failed"
                                            " operation",
                                            feature.get("language"),
                                        )
                                        then_key = lge.parse_keyword(
                                            "then", feature.get("language")
                                        )
                                        f_d = f"{then_key} {g_step_name}\n"
                                        l_f_d = f_d.format(
                                            run_count, relevance_key
                                        )
                                        cur_feature_array.append(l_f_d)

                                    if isinstance(scenario["steps"], list):
                                        for step in scenario["steps"]:
                                            keyword = step["keyword"]
                                            name = step["name"]
                                            cur_feature_array.append(
                                                f"    {keyword} {name}\n"
                                            )
                                    cur_feature_array.append("\n")
                                    scenario["status"] = "rerun"
                                    print(
                                        "rerun_feature_name",
                                        rerun_feature_name,
                                        "cur_feature_array",
                                        cur_feature_array,
                                    )
                                    file_helper.array_to_file(
                                        os.path.join(
                                            rerun_root_dir,
                                            f"{rerun_feature_name}.feature",
                                        ),
                                        cur_feature_array,
                                    )

                    file_helper.store_json_to_file_path(
                        report_json, file_path, "w"
                    )
            except Exception as e:
                raise Exception(
                    f"{file_item} Parsing is an error, innerError: {str(e)}"
                ) from e

    return sum_count, fail_count, fail_scenario_static, rerun_root_dir


def get_init_feature_array(index, language):
    """
    generate failed info
    """

    feature_des = []
    feature_des.append(f"# language: {language}\n\n")
    f_l = lge.parse_keyword("feature", language)
    r_f = lge.parse_glb_str("rerun failed scenario", language)
    feature_des.append(f"{f_l}:{r_f}{index}\n\n")

    return feature_des


def copy_behave_need_file(rerun_root_dir):
    """
    copy step.py and environment.py and other behave files to feature path
    """
    file_helper.create_dirs(os.path.join(rerun_root_dir, "steps"))
    file_helper.clear_dirs(os.path.join(rerun_root_dir, "steps"))
    shutil.copy(
        os.path.join(os.getcwd(), "features", "steps", "steps.py"),
        os.path.join(rerun_root_dir, "steps", "steps.py"),
    )
    shutil.copy(
        os.path.join(os.getcwd(), "features", "environment.py"),
        os.path.join(rerun_root_dir, "environment.py"),
    )
    shutil.copy(
        os.path.join(os.getcwd(), "features", "__init__.py"),
        os.path.join(rerun_root_dir, "__init__.py"),
    )


def set_rerun_info(user_data, gr):
    """
    set rerun info
    """
    last_fail_scenario_info_obj = {}
    # get rerun info, add into report
    if (
        "flybirdsAutoRerun" in user_data.keys()
        and user_data["flybirdsAutoRerun"] == "Yes"
    ):
        log.info("this is rerun case execute")
        rerun_info_array = user_data["flybirdsAutoRerunInfo"].split(",")
        if len(rerun_info_array) > 0:
            for item_rerun_info in rerun_info_array:
                rerun_info = item_rerun_info.strip()
                print("rerun_info", rerun_info)
                if (
                    "flybirdsAutoRerunInfo" in user_data.keys()
                    and os.path.exists(rerun_info)
                ):
                    try:
                        last_fail_scenario_info = (
                            file_helper.get_json_from_file(rerun_info)
                        )
                        last_fail_scenario_info_obj = dict(
                            last_fail_scenario_info_obj,
                            **last_fail_scenario_info,
                        )
                    except Exception as e:
                        log.info(
                            f"rerun case get exception info "
                            f"failed。innerError: {str(e)}"
                        )
            gr.set_value("rerunFailInfo", last_fail_scenario_info_obj)

    log.info(f"user data，count:{len(user_data)}")



# -*- coding: utf-8 -*-
"""
plugin load mannger
"""
import os
import sys
from imp import find_module, load_module, acquire_lock, release_lock
from flybirds.core.global_context import GlobalContext
from flybirds.core.plugin.plugin_proxy import PluginProxy
import flybirds.utils.flybirds_log as log


def append_prex(name, sub_pkg):
    """
    append default pkg
    """
    return "flybird.core.plugin.plugins." + sub_pkg + "." + name


def append_event_prex(name, sub_pkg):
    """
    append default pkg
    """
    return "flybird.core.plugin." + sub_pkg + "." + name


def append_config_prex(name, parent_pkg):
    """
    combine pkg and py file name as import module key
    """
    return parent_pkg + "." + name


def find_exsit_name(plugins, name, group):
    """
    get  name from plugins pth list
    """
    for index, item in enumerate(plugins):
        if (
                item[0] is not None
                and item[0] == name
                and item[3] is not None
                and item[3] == group
        ):
            return index + 1
    return 0


class PluginModule:
    """
    plugin  add find and remove func
    """

    name = "base"

    def __init__(self, plugins=(), config=None):
        if config is not None:
            self.c_config = config
        self.__plugins = []
        if plugins:
            self.add_plugins(plugins)

    def __iter__(self):
        return iter(self.plugins)

    def add_plugin(self, plug):
        """
        plugin add
        """
        self.__plugins.append(plug)

    def add_plugins(self, plugins):
        """
        plugin add list
        """
        for plug in plugins:
            self.add_plugin(plug)

    def del_plugin(self, plug):
        """
        plugin del
        """
        if plug in self.__plugins:
            self.__plugins.remove(plug)

    def del_plugins(self, plugins):
        """
        plugin del list
        """
        for plug in plugins:
            self.del_plugin(plug)

    def get_plugins(self, name=None):
        """
        plugin get list
        """
        plugins = []
        for plugin in self.__plugins:
            if name is None or plugin.name == name:
                plugins.append(plugin)
        return plugins

    def _load_plugin(self, plug):
        """
        regist plugin to list
        """
        loaded = False
        for plg in self.plugins:
            if plg.name == plug.name:
                loaded = True
                break
        if not loaded:
            self.add_plugin(plug)

    def _get_plugins(self):
        """
        property get func
        """
        return self.__plugins

    def _set_plugins(self, plugins):
        """
        property set func
        """
        self.__plugins = []
        self.add_plugins(plugins)

    plugins = property(
        _get_plugins,
        _set_plugins,
        None,
    )


# lock = Lock()


class DirectoryPluginManager(PluginModule):
    """
    Plugin manager that loads plugins from plugin directories.
    """

    name = "directory_loader"

    def __init__(self, plugins=(), config=None):
        default_directory = os.path.join(os.path.dirname(__file__), "plugins")
        if config is None:
            config = {}
        self.directories = config.get("directories", (default_directory,))
        PluginModule.__init__(self, plugins, config)

    def find_default_run_event_dir(self, plugins, group):
        """
        get default event plugin path list
        """
        base_dir = os.path.dirname(__file__)
        base_dir_list = (base_dir,)
        for dir_name in base_dir_list:
            try:
                dir_name = (
                    f"{dir_name}/event/"
                    f"{GlobalContext.platform}"
                )
                for f_p in os.listdir(dir_name):
                    if f_p.endswith(".py") and f_p != "__init__.py":
                        exsit_index = find_exsit_name(plugins, f_p[:-3], group)
                        if exsit_index == 0:
                            name = append_event_prex(
                                f_p[:-3],
                                "event."
                                f"{GlobalContext.platform}",
                            )
                            plugins.append((f_p[:-3], name, dir_name, group))
            except OSError:
                continue

        for dir_name in base_dir_list:
            try:
                dir_name = f"{dir_name}/event"
                for f_p in os.listdir(dir_name):
                    if f_p.endswith(".py") and f_p != "__init__.py":
                        exsit_index = find_exsit_name(plugins, f_p[:-3], group)
                        if exsit_index == 0:
                            event_pkg = "event"
                            plugins.append(
                                (
                                    f_p[:-3],
                                    append_event_prex(f_p[:-3], event_pkg),
                                    dir_name,
                                    group,
                                )
                            )
            except OSError:
                continue

    def find_default_dir(self, plugins, group):
        """
        get default platform plugin pth list
        """
        for d_n in self.directories:
            try:
                d_n = d_n + "/default/" + GlobalContext.platform
                for f_p in os.listdir(d_n):
                    if f_p.endswith(".py") and f_p != "__init__.py":
                        exsit_index = find_exsit_name(plugins, f_p[:-3], group)
                        if exsit_index == 0:
                            name = append_prex(
                                f_p[:-3],
                                f"{GlobalContext.active_plugin}"
                                f".{GlobalContext.platform}",
                            )
                            plugins.append((f_p[:-3], name, d_n, group))
            except OSError:
                log.info(f"do not exist path: {d_n}")
                continue

    @staticmethod
    def find_config_dir(plugins, group):
        """
        get user config platform pth list which level is higher than
        find_default_dir
        """
        plugin_info = getattr(GlobalContext, "plugin_info")
        if plugin_info is not None and plugin_info.__contains__(
                GlobalContext.platform
        ):
            active_plug_info = plugin_info.get(GlobalContext.platform)
            all_plugin = []

            for name, value in vars(GlobalContext).items():
                if (
                        value is not None
                        and not hasattr(value, "__call__")
                        and isinstance(value, PluginProxy)
                ):
                    all_plugin.append(name)

            for plugin_key in all_plugin:
                if (
                        active_plug_info is not None
                        and active_plug_info.__contains__(plugin_key)
                ):
                    py_config = active_plug_info.get(plugin_key)
                    if py_config is not None:
                        py_path = py_config.get("path")
                        py_ns = py_config.get("ns")
                        DirectoryPluginManager.add_plugin_path(
                            py_path, py_ns, plugin_key, plugins, group
                        )

    @staticmethod
    def add_plugin_path(py_path, py_ns, plugin_key, plugins, group):
        """
        add path to plugin
        """
        if (
                py_path is not None
                and py_path != ""
                and py_ns is not None
                and py_ns != ""
        ):
            py_name = os.path.basename(os.path.realpath(py_path))
            py_dir = os.path.dirname(os.path.realpath(py_path))
            if find_exsit_name(plugins, py_name[:-3], group) == 0:
                plugins.append(
                    (
                        plugin_key,
                        append_config_prex(py_name[:-3], py_ns),
                        py_dir,
                        group,
                    )
                )

    def load_plugins(self):
        """
        Load plugins by iterating files in plugin directories.
        """
        plugins = []
        self.find_default_run_event_dir(plugins, "event")
        DirectoryPluginManager.find_config_dir(plugins, "driver")
        self.find_default_dir(plugins, "driver")

        fh = None
        mod = None
        for (p_key, name, dir, group) in plugins:
            try:
                acquire_lock()
                fh, filename, desc = find_module(p_key, [dir])
                old = sys.modules.get(name)
                if old is not None:
                    del sys.modules[name]

                mod = load_module(name, fh, filename, desc)

                if hasattr(mod, "__open__"):
                    attrs = [getattr(mod, x) for x in mod.__open__]
                    for plug in attrs:
                        if hasattr(plug, "instantiation_timing") and \
                                plug.instantiation_timing == "plugin":
                            plugin_instance = plug
                        else:
                            plugin_instance = plug()
                        setattr(GlobalContext, p_key, plugin_instance)
            finally:
                if fh:
                    fh.close()
                release_lock()



# -*- coding: utf-8 -*-
"""
Element position api
"""
import time

from poco.exceptions import PocoNoSuchNodeException

import flybirds.core.plugin.plugins.default.ui_driver.poco.poco_ele as poco_ele
from flybirds.core.plugin.plugins.default.ui_driver.poco import poco_manage
import flybirds.utils.snippet as snippet
from flybirds.core.exceptions import FlybirdPositionChanging


def position_change(poco, select_str, optional, o_position):
    """
    determine whether the position of the element has changed within the
    specified time.
    """
    result = False
    timeout = optional["timeout"]
    current_wait_second = 1
    while (not result) and (timeout > 0):
        try:
            poco_target = poco_manage.create_poco_object_by_dsl(
                poco, select_str, optional
            )
            if poco_target.exists():
                t_position = poco_target.get_position()
                if not snippet.list_comparator(o_position, t_position):
                    result = True
            if result:
                break
        except Exception:
            pass
        time.sleep(current_wait_second)
        timeout -= current_wait_second
        current_wait_second += 1
    return result


def position_not_change(poco, select_str, optional, dur_time, verify_count):
    """
    determine the position of the element has not changed
    """
    poco_ele.wait_exists(poco, select_str, optional)
    prev_position = None

    log_time = dur_time * verify_count

    result = False
    while verify_count > 0 and (not result):
        verify_count -= 1
        try:
            if prev_position is None:
                prev_position = poco_manage.create_poco_object_by_dsl(
                    poco, select_str, optional
                ).get_position()
        except Exception:
            time.sleep(dur_time)
            continue

        time.sleep(dur_time)

        try:
            poco_target = poco_manage.create_poco_object_by_dsl(
                poco, select_str, optional
            )
            cur_position = poco_target.get_position()
            if snippet.list_comparator(prev_position, cur_position):
                result = True
            else:
                prev_position = cur_position
            if result:
                break
        except PocoNoSuchNodeException:
            result = True
        except Exception:
            prev_position = None
    if not result:
        message = "during {}s time, {} position is changing".format(
            log_time, select_str
        )
        raise FlybirdPositionChanging(message)



# -*- coding: utf-8 -*-
"""
launch init such as run args init
"""
from flybirds.core.launch_cycle.run_manage import RunManage


class LaunchInit:
    """
    launch init
    """

    name = "LaunchInit"
    order = 15

    @staticmethod
    def can(context):
        return True

    @staticmethod
    def run(context):
        """
        command set default value
        """
        option = {
            "cmd_str": None,
            "need_rerun_args": None,
            "report_dir_path": None,
            "is_html": None,
            "run_at": None,
            "no_args": None
        }
        if context is not None:
            context.update(option)
        else:
            context = option
        run_args = context["run_args"]
        if run_args is not None:
            context["cmd_str"] = run_args.get("cmd_str")
            context["need_rerun_args"] = run_args.get("need_rerun")
            context["report_dir_path"] = run_args.get("report_dir_path")
            context["no_args"] = False
            is_html = run_args.get("html")
            run_at = run_args.get("run_at")

            if is_html is not None and is_html is False:
                context["report_format"] = None
            else:
                context["report_format"] = "cucumber"
            if run_at is not None and run_at != "":
                context["run_at"] = run_at
            else:
                context["run_at"] = "local"


# add event to processor, launch init should be first one
RunManage.insert("before_run_processor", LaunchInit, 1)



# -*- coding: utf-8 -*-
"""
when behave start run hook will trigger this
"""
import traceback
from flybirds.utils import launch_helper
from flybirds.core.driver import ui_driver
import flybirds.core.global_resource as gr
import flybirds.utils.flybirds_log as log
from flybirds.core.global_context import GlobalContext


class OnBefore:  # pylint: disable=too-few-public-methods
    """
    before event
    """

    name = "OnBefore"
    order = 50

    @staticmethod
    def can(context):
        return True

    @staticmethod
    def init_ui_driver(context):
        """
        init device
        """
        # get deviceid
        device_id = gr.get_device_id()
        if device_id is not None:

            # get the globally defined poco object
            poco_instance = ui_driver.poco_init()
            gr.set_value("pocoInstance", poco_instance)
            context.poco_instance = poco_instance
            GlobalContext.ui_driver_instance = poco_instance
            log.info("poco object initialization completed")

            # get device screen size
            context.config_manage.device_info.screen_size = (
                ui_driver.air_bdd_screen_size(poco_instance)
            )
            log.info(
                f"device {context.config_manage.device_info.device_id} "
                f"get device"
                f" screen size {context.config_manage.device_info.screen_size}"
            )
        else:
            log.info(
                f"initialization device not obtained:{device_id} or platform"
            )

    @staticmethod
    def run(context):
        """
        event logical which try to connect device, init screen
        record, lunch login if it need
        """
        try:
            log.info("init device and screen config")
            OnBefore.init_ui_driver(context)
            # get the global object used to record the screen
            screen_record = GlobalContext.screen_record()
            gr.set_value("screenRecord", screen_record)
            context.screen_record = screen_record
            log.info("screen recording context initialization completed")
            if not screen_record.support:
                log.info("the device does not support screen recording")

            launch_helper.login()

            # startup method selection
            launch_helper.app_start("before_run_page")

        except Exception as init_error:
            log.info("global initialization error", traceback.format_exc())
            raise init_error
        # hook extend by tester
        before_all_extend = launch_helper.get_hook_file("before_all_extend")
        if before_all_extend is not None:
            before_all_extend(context)


class OnAfter:  # pylint: disable=too-few-public-methods
    """
    after event
    """

    name = "OnAfter"
    order = 100

    @staticmethod
    def can(context):
        return True

    @staticmethod
    def run(context):
        """
        close screen record
        """
        screen_record = gr.get_value("screenRecord")
        if screen_record is not None and hasattr(screen_record, "destroy"):
            screen_record.destroy()

        # hook extend by tester
        after_all_extend = launch_helper.get_hook_file("after_all_extend")
        if after_all_extend is not None:
            after_all_extend(context)


class OnScreenRecordRelease:
    """
    after event
    """

    name = "OnScreenRecordRelease"
    order = 10

    @staticmethod
    def can(context):
        return True

    @staticmethod
    def run(context):
        """
        close connector
        """
        try:
            screen_record = gr.get_value("screenRecord")
            if screen_record is not None and hasattr(screen_record, "destroy"):
                screen_record.destroy()
        except Exception as clear_error:
            log.info(f"clear screen record error :{str(clear_error)}")


class OnRelease:
    """
    after event
    """

    name = "OnRelease"
    order = 13

    @staticmethod
    def can(context):

        if GlobalContext.connector is not None:
            return True
        else:
            return False

    @staticmethod
    def run(context):
        """
        close connector
        """
        GlobalContext.connector.close()


# add event to global processor
var = GlobalContext.join("before_run_processor", OnBefore, 1)
var2 = GlobalContext.join("after_run_processor", OnScreenRecordRelease, 1)
var3 = GlobalContext.join("after_run_processor", OnRelease, 1)
var2 = GlobalContext.join("after_run_processor", OnAfter, 1)
